@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2019 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -46,6 +46,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <email/send.h>
 #include <developer/logic.h>
 #include <locale/translate.h>
+#include <editor/html2format.h>
+#include <book/create.h>
 
 
 void bible_logic_store_chapter (const string& bible, int book, int chapter, const string& usfm)
@@ -259,12 +261,16 @@ void bible_logic_import_resource (string bible, string resource)
 
 // This logs the change in the Bible text.
 // When $force is given, it records the change on clients also.
-void bible_logic_log_change (const string& bible, int book, int chapter, const string& usfm, string user, const string & summary, bool force)
+void bible_logic_log_change (const string& bible,
+                             int book, int chapter,
+                             const string& usfm,
+                             string user,
+                             const string & summary,
+                             [[maybe_unused]] bool force)
 {
 #ifdef HAVE_CLIENT
   if (!force) return;
 #endif
-  (void) force;
   
   Database_Bibles database_bibles;
   string existing_usfm = database_bibles.getChapter (bible, book, chapter);
@@ -296,8 +302,8 @@ void bible_logic_log_change (const string& bible, int book, int chapter, const s
       Filter_Text filter_text_new = Filter_Text (bible);
       filter_text_old.text_text = new Text_Text ();
       filter_text_new.text_text = new Text_Text ();
-      filter_text_old.addUsfmCode (existing_verse_usfm);
-      filter_text_new.addUsfmCode (verse_usfm);
+      filter_text_old.add_usfm_code (existing_verse_usfm);
+      filter_text_new.add_usfm_code (verse_usfm);
       filter_text_old.run (stylesheet);
       filter_text_new.run (stylesheet);
       string old_text = filter_text_old.text_text->get ();
@@ -362,13 +368,13 @@ void bible_logic_kick_unsent_data_timer ()
   if (Database_Config_General::getUnsentBibleDataTime () != 0) return;
   
   // Stamp with the current time.
-  Database_Config_General::setUnsentBibleDataTime (filter_date_seconds_since_epoch ());
+  Database_Config_General::setUnsentBibleDataTime (filter::date::seconds_since_epoch ());
 }
 
 
 void bible_logic_kick_unreceived_data_timer ()
 {
-  Database_Config_General::setUnreceivedBibleDataTime (filter_date_seconds_since_epoch ());
+  Database_Config_General::setUnreceivedBibleDataTime (filter::date::seconds_since_epoch ());
 }
 
 
@@ -384,7 +390,7 @@ string bible_logic_unsent_unreceived_data_warning ()
   int data_time = Database_Config_General::getUnreceivedBibleDataTime ();
   // A value of 0 means that it is not relevant.
   if (data_time) {
-    int now = filter_date_seconds_since_epoch ();
+    int now = filter::date::seconds_since_epoch ();
     // Unreceived data warning after four days.
     data_time += (4 * 24 * 3600);
     if (now > data_time) {
@@ -401,7 +407,7 @@ string bible_logic_unsent_unreceived_data_warning ()
   data_time = Database_Config_General::getUnsentBibleDataTime ();
   // A value of 0 means that there's no pending data.
   if (data_time) {
-    int now = filter_date_seconds_since_epoch ();
+    int now = filter::date::seconds_since_epoch ();
     // Unsent data warning after four days.
     data_time += (4 * 24 * 3600);
     if (now > data_time) {
@@ -426,42 +432,87 @@ void bible_logic_merge_irregularity_mail (vector <string> users, vector <Merge_C
   
   for (auto & conflict : conflicts) {
     
+    // Add the passage to the subject.
+    string newsubject = conflict.subject;
+    if (conflict.book) newsubject.append (" | " + filter_passage_display (conflict.book, conflict.chapter, ""));
+    
     // Create the body of the email.
     xml_document document;
     xml_node node;
     node = document.append_child ("h3");
-    node.text ().set (conflict.subject.c_str());
-    
+    node.text ().set (newsubject.c_str());
+
+    // Storage of the changes the user sent, and the result that was saved, in raw USFM, per verse.
+    vector <string> change_usfm;
+    vector <string> result_usfm;
+
+    // Go through all verses available in the USFM,
+    // and make a record for each verse,
+    // where the USFM differs between the change that the user made and the result that was saved.
+    vector <int> verses = usfm_get_verse_numbers (conflict.result);
+    for (auto verse : verses) {
+      string change = usfm_get_verse_text (conflict.change, verse);
+      string result = usfm_get_verse_text (conflict.result, verse);
+      // When there's no change in the verse, skip it.
+      if (change == result) continue;
+      // Record the difference.
+      change_usfm.push_back (change);
+      result_usfm.push_back (result);
+    }
+
     // Add some information for the user.
     node = document.append_child ("p");
-    node.text ().set ("While merging the text, something unusual was detected.");
+    node.text ().set ("You sent changes to the Cloud. The changes were merged with other changes already in the Cloud. The crossed out parts below could not be merged. They were replaced with the bold text below. You may want to check the changes or resend them.");
+
+    // Go over each verse where the change differs from the resulting text that was saved.
+    // For each verse, outline the difference between the change and the result.
+    // Clearly maark it up for the user.
+    // The purpose is that the user immediately can see what happened,
+    // and whether and how to correct it.
+    for (size_t i = 0; i < change_usfm.size(); i++) {
+      node = document.append_child ("p");
+      string modification = filter_diff_diff (change_usfm[i], result_usfm[i]);
+      // Add raw html to the email's text buffer.
+      node.append_buffer (modification.c_str (), modification.size ());
+    }
+    
+    // Add some information for the user.
+    document.append_child ("hr");
+    document.append_child ("br");
+    xml_node div_node;
+    div_node = document.append_child ("div");
+    // Disabled: https://github.com/bibledit/cloud/issues/401
+    // div_node.append_attribute ("style") = "font-size: 30%";
+
+    node = div_node.append_child ("p");
+    node.text ().set ("Full details follow below.");
     
     // Add the base text.
-    document.append_child ("br");
-    node = document.append_child ("p");
-    node.text ().set ("Base text");
-    node = document.append_child ("pre");
+    div_node.append_child ("br");
+    node = div_node.append_child ("p");
+    node.text ().set ("Base text loaded in your editor");
+    node = div_node.append_child ("pre");
     node.text ().set (conflict.base.c_str ());
     
     // Add the changed text.
-    document.append_child ("br");
-    node = document.append_child ("p");
-    node.text ().set ("Changed text");
-    node = document.append_child ("pre");
+    div_node.append_child ("br");
+    node = div_node.append_child ("p");
+    node.text ().set ("Changed text by you");
+    node = div_node.append_child ("pre");
     node.text ().set (conflict.change.c_str ());
     
     // Add the existing text.
-    document.append_child ("br");
-    node = document.append_child ("p");
-    node.text ().set ("Existing text");
-    node = document.append_child ("pre");
+    div_node.append_child ("br");
+    node = div_node.append_child ("p");
+    node.text ().set ("Existing text in the Cloud");
+    node = div_node.append_child ("pre");
     node.text ().set (conflict.prioritized_change.c_str ());
     
     // Add the merge result.
-    document.append_child ("br");
-    node = document.append_child ("p");
-    node.text ().set ("The text that was actually saved to the chapter");
-    node = document.append_child ("pre");
+    div_node.append_child ("br");
+    node = div_node.append_child ("p");
+    node.text ().set ("The text that was actually saved to the chapter in the Cloud");
+    node = div_node.append_child ("pre");
     node.text ().set (conflict.result.c_str ());
     
     // Convert the document to a string.
@@ -471,21 +522,28 @@ void bible_logic_merge_irregularity_mail (vector <string> users, vector <Merge_C
     
     // Schedule the mail for sending to the user(s).
     for (auto user : users) {
-      email_schedule (user, conflict.subject, html);
+      email_schedule (user, newsubject, html);
     }
   }
 }
 
 
-void bible_logic_unsafe_save_mail (const string & message, const string & explanation, const string & user, const string & usfm)
+void bible_logic_unsafe_save_mail (string subject, const string & explanation,
+                                   const string & user,
+                                   const string & usfm,
+                                   int book, int chapter)
 {
-  if (message.empty ()) return;
-  
+  if (subject.empty ()) return;
+
+  // Add the passage to the subject for extra clarity.
+  // https://github.com/bibledit/cloud/issues/676
+  subject.append (" | " + filter_passage_display (book, chapter, ""));
+
   // Create the body of the email.
   xml_document document;
   xml_node node;
   node = document.append_child ("h3");
-  node.text ().set (message.c_str());
+  node.text ().set (subject.c_str());
   
   // Add some information for the user.
   node = document.append_child ("p");
@@ -504,15 +562,18 @@ void bible_logic_unsafe_save_mail (const string & message, const string & explan
   string html = output.str ();
   
   // Schedule the mail for sending to the user.
-  email_schedule (user, message, html);
+  email_schedule (user, subject, html);
 }
 
 
 // This function sends an email
 // if the USFM received from the client
 // does not match the USFM that gets stored on the server.
-void bible_logic_client_receive_merge_mail (const string & bible, int book, int chapter, const string & user,
-                                            const string & client_old, const string & client_new, const string & server)
+void bible_logic_client_receive_merge_mail (const string & bible, int book, int chapter,
+                                            const string & user,
+                                            const string & client_old,
+                                            const string & client_new,
+                                            const string & server)
 {
   // No difference: Done.
   if (client_old == server) return;
@@ -539,8 +600,10 @@ void bible_logic_client_receive_merge_mail (const string & bible, int book, int
   // No differences found: Done.
   if (client_diff.empty ()) return;
   
+  // Add the passage to the subject.
   string subject = "Saved Bible text was merged";
-  
+  subject.append (" | " + filter_passage_display (book, chapter, ""));
+
   // Create the body of the email.
   xml_document document;
   xml_node node;
@@ -614,8 +677,9 @@ void bible_logic_client_mail_pending_bible_updates (string user)
         if (newusfm == oldusfm) continue;
         if (newusfm.empty ()) continue;
 
-        string location = bible + " " + filter_passage_display (book, chapter, "");
-        string subject = "Discarded text update " + location;
+        // Add the passage to the subject.
+        string subject = "Discarded text update";
+        subject.append (" | " + filter_passage_display (book, chapter, ""));
         
         // Create the body of the email.
         xml_document document;
@@ -637,6 +701,7 @@ void bible_logic_client_mail_pending_bible_updates (string user)
         
         // Add the passage.
         node = document.append_child ("p");
+        string location = bible + " " + filter_passage_display (book, chapter, "") +  ".";
         node.text ().set (location.c_str ());
         
         // Add the text.
@@ -657,7 +722,8 @@ void bible_logic_client_mail_pending_bible_updates (string user)
 }
 
 
-void bible_logic_client_no_write_access_mail (const string & bible, int book, int chapter, const string & user,
+void bible_logic_client_no_write_access_mail (const string & bible, int book, int chapter,
+                                              const string & user,
                                               const string & oldusfm, const string & newusfm)
 {
   // No difference: Done.
@@ -682,8 +748,10 @@ void bible_logic_client_no_write_access_mail (const string & bible, int book, in
   // No differences found: Done.
   if (client_new_diff.empty ()) return;
   
+  // Add the passage to the subject.
   string subject = "No write access while sending Bible text";
-  
+  subject.append (" | " + filter_passage_display (book, chapter, ""));
+
   // Create the body of the email.
   xml_document document;
   xml_node node;
@@ -725,12 +793,11 @@ void bible_logic_client_no_write_access_mail (const string & bible, int book, in
 }
 
 
-void bible_logic_recent_save_email (const string & bible, int book, int chapter, int verse,
+void bible_logic_recent_save_email (const string & bible,
+                                    int book, int chapter,
                                     const string & user,
                                     const string & old_usfm, const string & new_usfm)
 {
-  (void) verse;
-  
   vector <string> old_verses;
   vector <string> new_verses;
 
@@ -751,8 +818,10 @@ void bible_logic_recent_save_email (const string & bible, int book, int chapter,
   // No differences found: Done.
   if (new_verses.empty ()) return;
 
+  // Add the passage to the subject.
   string subject = translate ("Check whether Bible text was saved");
-  
+  subject.append (" | " + filter_passage_display (book, chapter, ""));
+
   // Create the body of the email.
   xml_document document;
   xml_node node;
@@ -764,16 +833,19 @@ void bible_logic_recent_save_email (const string & bible, int book, int chapter,
   string information;
   information.append (translate ("Bibledit saved the Bible text below."));
   information.append (" ");
-  information.append (translate ("But it found that other Bible text was saved to the same chapter."));
-  information.append (" ");
-  information.append (translate ("This may have been done by you or by someone else."));
+  information.append (translate ("But Bibledit is not entirely sure that all went well."));
   information.append (" ");
   information.append (translate ("You may want to check whether the Bible text was saved correctly."));
+  information.append (" ");
+  information.append (translate ("The text in bold was added."));
+  information.append (" ");
+  information.append (translate ("The text in strikethrough was removed."));
   node.text ().set (information.c_str());
   node = document.append_child ("p");
   string location = bible + " " + filter_passage_display (book, chapter, "") +  ".";
   node.text ().set (location.c_str ());
 
+  bool differences_found = false;
   for (unsigned int i = 0; i < new_verses.size(); i++) {
     Filter_Text filter_text_old = Filter_Text (bible);
     Filter_Text filter_text_new = Filter_Text (bible);
@@ -781,8 +853,8 @@ void bible_logic_recent_save_email (const string & bible, int book, int chapter,
     filter_text_new.html_text_standard = new Html_Text (translate("Bible"));
     filter_text_old.text_text = new Text_Text ();
     filter_text_new.text_text = new Text_Text ();
-    filter_text_old.addUsfmCode (old_verses[i]);
-    filter_text_new.addUsfmCode (new_verses[i]);
+    filter_text_old.add_usfm_code (old_verses[i]);
+    filter_text_new.add_usfm_code (new_verses[i]);
     filter_text_old.run (styles_logic_standard_sheet());
     filter_text_new.run (styles_logic_standard_sheet());
     string old_text = filter_text_old.text_text->get ();
@@ -792,14 +864,327 @@ void bible_logic_recent_save_email (const string & bible, int book, int chapter,
       string modification = filter_diff_diff (old_text, new_text);
       string fragment = /* convert_to_string (verse) + " " + */ modification;
       node.append_buffer (fragment.c_str (), fragment.size ());
+      differences_found = true;
     }
   }
+  // If no differences were found, bail out.
+  // This also handles differences in spacing.
+  // If the differences consist of whitespace only, bail out here.
+  // See issue https://github.com/bibledit/cloud/issues/413
+  if (!differences_found) return;
   
   // Convert the document to a string.
   stringstream output;
   document.print (output, "", format_raw);
   string html = output.str ();
-  
+
   // Schedule the mail for sending to the user.
   email_schedule (user, subject, html);
 }
+
+
+void bible_logic_optional_merge_irregularity_email (const string & bible, int book, int chapter,
+                                                    const string & user,
+                                                    const string & ancestor_usfm,
+                                                    const string & edited_usfm,
+                                                    const string & merged_usfm)
+{
+  // If the merged edited USFM is the same as the edited USFM,
+  // that means that the user's changes will get saved to the chapter.
+  if (edited_usfm == merged_usfm) return;
+  // But if the merged edited USFM differs from the original edited USFM,
+  // more checks need to be done to be sure that the user's edits made it.
+
+  // Add the passage to the subject.
+  string subject = translate ("Check whether Bible text was saved");
+  subject.append (" | " + filter_passage_display (book, chapter, ""));
+
+  // Create the body of the email.
+  xml_document document;
+  xml_node node;
+  node = document.append_child ("h3");
+  node.text ().set (subject.c_str());
+  
+  // Add some information for the user.
+  node = document.append_child ("p");
+  string information;
+  information.append (translate ("Bibledit saved the Bible text below."));
+  information.append (" ");
+  information.append (translate ("But Bibledit is not entirely sure that all went well."));
+  information.append (" ");
+  information.append (translate ("You may want to check whether the Bible text was saved correctly."));
+  information.append (" ");
+  information.append (translate ("The text in bold was added."));
+  information.append (" ");
+  information.append (translate ("The text in strikethrough was removed."));
+  node.text ().set (information.c_str());
+  node = document.append_child ("p");
+  string location = bible + " " + filter_passage_display (book, chapter, "") +  ".";
+  node.text ().set (location.c_str ());
+
+  bool anomalies_found = false;
+
+  // Go through all verses available in the USFM,
+  // and check the differences for each verse.
+  vector <int> verses = usfm_get_verse_numbers (merged_usfm);
+  for (auto verse : verses) {
+    string ancestor_verse_usfm = usfm_get_verse_text (ancestor_usfm, verse);
+    string edited_verse_usfm = usfm_get_verse_text (edited_usfm, verse);
+    string merged_verse_usfm = usfm_get_verse_text (merged_usfm, verse);
+    // There's going to be a check to find out that all the changes the user made,
+    // are available among the changes resulting from the merge.
+    // If all the changes are there, all is good.
+    // If not, then email the user about this.
+    bool anomaly_found = false;
+    vector <string> user_removals, user_additions, merged_removals, merged_additions;
+    filter_diff_diff (ancestor_verse_usfm, edited_verse_usfm, &user_removals, &user_additions);
+    filter_diff_diff (ancestor_verse_usfm, merged_verse_usfm, &merged_removals, &merged_additions);
+    for (auto user_removal : user_removals) {
+      if (!in_array (user_removal, merged_removals)) anomaly_found = true;
+    }
+    for (auto user_addition : user_additions) {
+      if (!in_array (user_addition, merged_additions)) anomaly_found = true;
+    }
+    if (!anomaly_found) continue;
+    Filter_Text filter_text_ancestor = Filter_Text (bible);
+    Filter_Text filter_text_edited = Filter_Text (bible);
+    Filter_Text filter_text_merged = Filter_Text (bible);
+    filter_text_ancestor.html_text_standard = new Html_Text (translate("Bible"));
+    filter_text_edited.html_text_standard = new Html_Text (translate("Bible"));
+    filter_text_merged.html_text_standard = new Html_Text (translate("Bible"));
+    filter_text_ancestor.text_text = new Text_Text ();
+    filter_text_edited.text_text = new Text_Text ();
+    filter_text_merged.text_text = new Text_Text ();
+    filter_text_ancestor.add_usfm_code (ancestor_verse_usfm);
+    filter_text_edited.add_usfm_code (edited_verse_usfm);
+    filter_text_merged.add_usfm_code (merged_verse_usfm);
+    filter_text_ancestor.run (styles_logic_standard_sheet());
+    filter_text_edited.run (styles_logic_standard_sheet());
+    filter_text_merged.run (styles_logic_standard_sheet());
+    string ancestor_text = filter_text_ancestor.text_text->get ();
+    string edited_text = filter_text_edited.text_text->get ();
+    string merged_text = filter_text_merged.text_text->get ();
+    string modification_edited = filter_diff_diff (ancestor_text, edited_text);
+    string modification_saved = filter_diff_diff (ancestor_text, merged_text);
+    // If the edited and saved modifications are the same, do not email this.
+    // This keeps changes in footnotes and cross references out from consideration.
+    if (modification_saved == modification_edited) continue;
+    // Add labels to the modifications.
+    modification_edited.insert(0, translate ("You edited:") + " ");
+    modification_saved.insert(0, translate ("Bibledit saved:") + " ");
+    // Add the modifications to the email body.
+    node = document.append_child ("p");
+    node.append_buffer (modification_edited.c_str (), modification_edited.size ());
+    node = document.append_child ("p");
+    node.append_buffer (modification_saved.c_str (), modification_saved.size ());
+    anomalies_found = true;
+  }
+
+  // If no differences were found, bail out.
+  // This also handles differences in spacing.
+  // If the differences consist of whitespace only, bail out here.
+  // See issue https://github.com/bibledit/cloud/issues/413
+  if (!anomalies_found) return;
+  
+  // Convert the document to a string.
+  stringstream output;
+  document.print (output, "", format_raw);
+  string html = output.str ();
+
+  // Schedule the mail for sending to the user.
+  email_schedule (user, subject, html);
+}
+
+
+const char * bible_logic_insert_operator ()
+{
+  return "i";
+}
+const char * bible_logic_delete_operator ()
+{
+  return "d";
+}
+const char * bible_logic_format_paragraph_operator ()
+{
+  return "p";
+}
+const char * bible_logic_format_character_operator ()
+{
+  return "c";
+}
+
+// There are three containers with updating information.
+// The function condenses this updating information.
+// This condensed information works better for the Quill editor.
+void bible_logic_condense_editor_updates (const vector <int> & positions_in,
+                                          const vector <int> & sizes_in,
+                                          const vector <bool> & additions_in,
+                                          const vector <string> & content_in,
+                                          vector <int> & positions_out,
+                                          vector <int> & sizes_out,
+                                          vector <string> & operators_out,
+                                          vector <string> & content_out)
+{
+  positions_out.clear();
+  sizes_out.clear();
+  operators_out.clear();
+  content_out.clear();
+  
+  // https://stackoverflow.com/questions/40492414/why-does-stdnumeric-limitslong-longmax-fail
+  int previous_position = (numeric_limits<int>::min)();
+  bool previous_addition = false;
+  string previous_character = string();
+  for (size_t i = 0; i < positions_in.size(); i++) {
+    int position     = positions_in[i];
+    int size         = sizes_in[i];
+    bool addition    = additions_in[i];
+    string character = content_in[i].substr (0, 1);
+    string format = content_in[i].substr (1);
+
+    // The following situation occurs when changing the style of a paragraph.
+    // Like for example changing a paragraph style from "p" to "s".
+    // The current sequence for this change is:
+    // 1. Delete a new line at a known position.
+    // 2. Insert a new line at the same position, with a given format.
+    // Condense this as follows:
+    // 1. Apply the given paragraph format to the given position.
+    // Without condensing this, the following would happen:
+    // 1. Delete the new line before the second line.
+    // Result: The first line gets the format of the second line.
+    // 2. Insert the new line before the second line again.
+    // 3. Appy formatting to the second line.
+    // The net result is that the first line remains with the paragraph format of the second line.
+    bool newlineflag = addition && !previous_addition && (character == "\n") && (character == previous_character) && (position == previous_position);
+    if (newlineflag) {
+      // Remove the previous "delete new line".
+      positions_out.pop_back();
+      sizes_out.pop_back();
+      operators_out.pop_back();
+      content_out.pop_back();
+      // Add the paragraph format operation data.
+      positions_out.push_back(position);
+      sizes_out.push_back(size);
+      operators_out.push_back(bible_logic_format_paragraph_operator());
+      content_out.push_back(format);
+    } else {
+      positions_out.push_back(position);
+      sizes_out.push_back(size);
+      if (addition) operators_out.push_back(bible_logic_insert_operator());
+      else operators_out.push_back(bible_logic_delete_operator());
+      content_out.push_back(character + format);
+    }
+
+    // Store data for the next iteration.
+    previous_position = position;
+    previous_addition = addition;
+    previous_character = character;
+  }
+  
+}
+
+
+void bible_logic_html_to_editor_updates (const string & editor_html,
+                                         const string & server_html,
+                                         vector <int> & positions,
+                                         vector <int> & sizes,
+                                         vector <string> & operators,
+                                         vector <string> & content)
+{
+  // Clear outputs.
+  positions.clear();
+  sizes.clear();
+  operators.clear();
+  content.clear();
+  
+  // Convert the html to formatted text.
+  Editor_Html2Format editor_format;
+  Editor_Html2Format server_format;
+  editor_format.load (editor_html);
+  server_format.load (server_html);
+  editor_format.run ();
+  server_format.run ();
+
+  // Convert the formatted text fragments to formatted UTF-8 characters.
+  vector <string> editor_formatted_character_content;
+  for (size_t i = 0; i < editor_format.texts.size(); i++) {
+    string text = editor_format.texts[i];
+    string format = editor_format.formats[i];
+    size_t length = unicode_string_length (text);
+    for (size_t pos = 0; pos < length; pos++) {
+      string utf8_character = unicode_string_substr (text, pos, 1);
+      editor_formatted_character_content.push_back (utf8_character + format);
+    }
+  }
+  vector <string> server_formatted_character_content;
+  vector <string> server_utf8_characters;
+  for (size_t i = 0; i < server_format.texts.size(); i++) {
+    string text = server_format.texts[i];
+    string format = server_format.formats[i];
+    size_t length = unicode_string_length (text);
+    for (size_t pos = 0; pos < length; pos++) {
+      string utf8_character = unicode_string_substr (text, pos, 1);
+      server_formatted_character_content.push_back (utf8_character + format);
+      server_utf8_characters.push_back(utf8_character);
+    }
+  }
+
+  // Find the differences between the two sets of content.
+  vector <int> positions_diff;
+  vector <int> sizes_diff;
+  vector <bool> additions_diff;
+  vector <string> content_diff;
+  int new_line_diff_count;
+  filter_diff_diff_utf16 (editor_formatted_character_content, server_formatted_character_content,
+                          positions_diff, sizes_diff, additions_diff, content_diff, new_line_diff_count);
+
+  // Condense the differences a bit and render them to another format.
+  bible_logic_condense_editor_updates (positions_diff, sizes_diff, additions_diff, content_diff,
+                                       positions, sizes, operators, content);
+
+  // Problem description:
+  // User action: Remove the new line at the end of the current paragraph.
+  // Result: The current paragraph takes the style of the next paragraph.
+  // User actions: While removing notes, this goes wrong.
+  // Solution:
+  // If there's new line(s) added or removed, apply all paragraph styles again.
+  if (new_line_diff_count) {
+    int position = 0;
+    for (size_t i = 0; i < server_utf8_characters.size(); i++) {
+      int size = (int)convert_to_u16string (server_utf8_characters[i]).length();
+      if (server_utf8_characters[i] == "\n") {
+        positions.push_back(position);
+        sizes.push_back(size);
+        operators.push_back(bible_logic_format_paragraph_operator());
+        content.push_back(server_formatted_character_content[i].substr (1));
+      }
+      position += size;
+    }
+  }
+  
+}
+
+
+void bible_logic_create_empty_bible (const string & name)
+{
+  Database_Logs::log (translate("Creating Bible") + " " + name);
+  
+  // Remove and create the empty Bible.
+  Database_Bibles database_bibles;
+  database_bibles.deleteBible (name);
+  database_bibles.createBible (name);
+  
+  // Remove index for the Bible.
+  search_logic_delete_bible (name);
+
+  // Create books with blank verses for the OT and NT.
+  vector <int> books = Database_Books::getIDs ();
+  for (auto book : books) {
+    string type = Database_Books::getType (book);
+    if ((type == "ot") || (type == "nt")) {
+      vector <string> feedback;
+      book_create (name, book, -1, feedback);
+    }
+  }
+  
+  Database_Logs::log (translate("Created:") + " " + name);
+}
