@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <user/account.h>
 #include <manage/index.h>
 #include <manage/users.h>
+#include <manage/accounts.h>
 #include <manage/exports.h>
 #include <manage/hyphenation.h>
 #include <manage/write.h>
@@ -127,6 +128,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <resource/bbgateway.h>
 #include <resource/studylight.h>
 #include <resource/unload.h>
+#include <locale/translate.h>
 #include <mapping/index.h>
 #include <mapping/map.h>
 #include <notes/index.h>
@@ -190,24 +192,29 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <filter/string.h>
 #include <journal/logic.h>
 #include <nmt/index.h>
-#include <edit2/index.h>
-#include <edit2/edit.h>
 #include <editor/id.h>
-#include <edit2/load.h>
-#include <edit2/save.h>
 #include <editor/style.h>
-#include <edit2/preview.h>
-#include <edit2/position.h>
-#include <edit2/navigate.h>
-#include <edit2/update.h>
+#include <edit/update.h>
 #include <editone2/index.h>
 #include <editone2/load.h>
 #include <editone2/save.h>
 #include <editone2/verse.h>
 #include <editone2/update.h>
+#include <read/index.h>
+#include <read/load.h>
+#include <read/verse.h>
+#include <resource/divider.h>
+#include <session/confirm.h>
+#include <resource/comparative9edit.h>
+#include <resource/comparative1edit.h>
+#include <developer/logic.h>
+#include <developer/delay.h>
+#include <images/index.h>
+#include <images/view.h>
+#include <images/fetch.h>
 
 
-// Internal function to check whether a request coming from the browser is considered secure enough.
+// Check whether a request coming from the browser is considered secure enough.
 // It returns true if the security is okay.
 bool browser_request_security_okay (Webserver_Request * request)
 {
@@ -232,19 +239,20 @@ bool browser_request_security_okay (Webserver_Request * request)
 // it decides which functions to call to obtain the response.
 void bootstrap_index (void * webserver_request)
 {
-  if (config_logic_log_incoming_connections ()) {
-    journal_logic_log_incoming_connection (webserver_request);
+  shared_ptr<Developer_Logic_Tracer> developer_logic_tracer = nullptr;
+  if (config_globals_log_network) {
+    developer_logic_tracer = make_shared<Developer_Logic_Tracer>(webserver_request);
   }
 
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   
   // Record the POST request made to the web server.
   // This can be used for debugging.
   /*
   if (!request->post.empty () && config_logic_demo_enabled ()) {
     string contents;
-    int seconds = filter_date_seconds_since_epoch ();
-    string rfc822time = filter_date_rfc822 (seconds);
+    int seconds = filter::date::seconds_since_epoch ();
+    string rfc822time = filter::date::rfc822 (seconds);
     contents.append (rfc822time + "\n");
     contents.append (request->get + "\n");
     string query;
@@ -266,22 +274,24 @@ void bootstrap_index (void * webserver_request)
   string extension = filter_url_get_extension (request->get);
   string url = request->get.substr (1);
   
-  // Serve graphics, stylesheets, JavaScript, fonts, with direct streaming for low memory usage.
+  
+  // Serve graphics, stylesheets, JavaScript, fonts, and so on.
+  // Use direct streaming for low memory usage.
   if (   (extension == "ico")
-      || (extension == "png")
-      || (extension == "gif")
-      || (extension == "jpg")
+      || (filter_url_is_image (extension))
       || (extension == "css")
       || (extension == "js")
       || (Fonts_Logic::isFont (extension))
       || (extension == "sh")
-      || (extension == "svg")
       || (extension == "map")
+      || (extension == "webmanifest")
       ) {
     http_stream_file (request, true);
     return;
   }
 
+  // check_user_localization_preference (request);
+
   if ((url == resource_imagefetch_url ()) && resource_imagefetch_acl (request)) {
     request->reply = resource_imagefetch (request);
     return;
@@ -678,7 +688,12 @@ void bootstrap_index (void * webserver_request)
     request->reply = manage_users (request);
     return;
   }
-  
+
+  if ((url == manage_accounts_url ()) && browser_request_security_okay (request) && manage_accounts_acl (request)) {
+    request->reply = manage_accounts (request);
+    return;
+  }
+
   if ((url == manage_index_url ()) && browser_request_security_okay (request) && manage_index_acl (request)) {
     request->reply = manage_index (request);
     return;
@@ -688,7 +703,8 @@ void bootstrap_index (void * webserver_request)
     request->reply = system_index (request);
     return;
   }
-  
+
+
   if ((url == email_index_url ()) && browser_request_security_okay (request) && email_index_acl (request)) {
     request->reply = email_index (request);
     return;
@@ -1122,33 +1138,33 @@ void bootstrap_index (void * webserver_request)
     return;
   }
 
-  if ((url == edit2_index_url ()) && browser_request_security_okay (request) && edit2_index_acl (request)) {
-    request->reply = edit2_index (request);
+  if ((url == edit_index_url ()) && browser_request_security_okay (request) && edit_index_acl (request)) {
+    request->reply = edit_index (request);
     return;
   }
   
-  if ((url == edit2_position_url ()) && browser_request_security_okay (request) && edit2_position_acl (request)) {
-    request->reply = edit2_position (request);
+  if ((url == edit_position_url ()) && browser_request_security_okay (request) && edit_position_acl (request)) {
+    request->reply = edit_position (request);
     return;
   }
 
-  if ((url == edit2_navigate_url ()) && browser_request_security_okay (request) && edit2_navigate_acl (request)) {
-    request->reply = edit2_navigate (request);
+  if ((url == edit_navigate_url ()) && browser_request_security_okay (request) && edit_navigate_acl (request)) {
+    request->reply = edit_navigate (request);
     return;
   }
   
-  if ((url == edit2_preview_url ()) && browser_request_security_okay (request) && edit2_preview_acl (request)) {
-    request->reply = edit2_preview (request);
+  if ((url == edit_preview_url ()) && browser_request_security_okay (request) && edit_preview_acl (request)) {
+    request->reply = edit_preview (request);
     return;
   }
   
-  if ((url == edit2_edit_url ()) && browser_request_security_okay (request) && edit2_edit_acl (request)) {
-    request->reply = edit2_edit (request);
+  if ((url == edit_edit_url ()) && browser_request_security_okay (request) && edit_edit_acl (request)) {
+    request->reply = edit_edit (request);
     return;
   }
 
-  if ((url == edit2_update_url ()) && browser_request_security_okay (request) && edit2_update_acl (request)) {
-    request->reply = edit2_update (request);
+  if ((url == edit_update_url ()) && browser_request_security_okay (request) && edit_update_acl (request)) {
+    request->reply = edit_update (request);
     return;
   }
 
@@ -1157,13 +1173,13 @@ void bootstrap_index (void * webserver_request)
     return;
   }
   
-  if ((url == edit2_load_url ()) && browser_request_security_okay (request) && edit2_load_acl (request)) {
-    request->reply = edit2_load (request);
+  if ((url == edit_load_url ()) && browser_request_security_okay (request) && edit_load_acl (request)) {
+    request->reply = edit_load (request);
     return;
   }
   
-  if ((url == edit2_save_url ()) && browser_request_security_okay (request) && edit2_save_acl (request)) {
-    request->reply = edit2_save (request);
+  if ((url == edit_save_url ()) && browser_request_security_okay (request) && edit_save_acl (request)) {
+    request->reply = edit_save (request);
     return;
   }
   
@@ -1197,6 +1213,61 @@ void bootstrap_index (void * webserver_request)
     return;
   }
 
+  if ((url == read_index_url ()) && browser_request_security_okay (request) && read_index_acl (request)) {
+    request->reply = read_index (request);
+    return;
+  }
+
+  if ((url == read_load_url ()) && browser_request_security_okay (request) && read_load_acl (request)) {
+    request->reply = read_load (request);
+    return;
+  }
+
+  if ((url == read_verse_url ()) && browser_request_security_okay (request) && read_verse_acl (request)) {
+    request->reply = read_verse (request);
+    return;
+  }
+
+  if ((url == resource_divider_url ()) && browser_request_security_okay (request) && resource_divider_acl (request)) {
+    request->reply = resource_divider (request);
+    return;
+  }
+
+  if ((url == session_confirm_url ()) && browser_request_security_okay (request) && session_confirm_acl (request)) {
+    request->reply = session_confirm (request);
+    return;
+  }
+  
+  if ((url == resource_comparative9edit_url ()) && browser_request_security_okay (request) && resource_comparative9edit_acl (request)) {
+    request->reply = resource_comparative9edit (request);
+    return;
+  }
+
+  if ((url == resource_comparative1edit_url ()) && browser_request_security_okay (request) && resource_comparative1edit_acl (request)) {
+    request->reply = resource_comparative1edit (request);
+    return;
+  }
+
+  if ((url == developer_delay_url ()) && developer_delay_acl (request)) {
+    request->reply = developer_delay (request);
+    return;
+  }
+
+  if ((url == images_index_url ()) && browser_request_security_okay (request) && images_index_acl (request)) {
+    request->reply = images_index (request);
+    return;
+  }
+
+  if ((url == images_view_url ()) && browser_request_security_okay (request) && images_view_acl (request)) {
+    request->reply = images_view (request);
+    return;
+  }
+
+  if ((url == images_fetch_url ()) && browser_request_security_okay (request) && images_fetch_acl (request)) {
+    request->reply = images_fetch (request);
+    return;
+  }
+
   // Forward the browser to the default home page.
   redirect_browser (request, index_index_url ());
 }
