@@ -146,7 +146,7 @@ string changes_change (void * webserver_request)
     int timestap = database_notes.get_modified (note);
     timestamps.push_back (timestap);
   }
-  quick_sort (timestamps, notes, 0, static_cast<int>(notes.size ()));
+  quick_sort (timestamps, notes, 0, static_cast <unsigned int> (notes.size ()));
   reverse (notes.begin(), notes.end());
   
   
@@ -177,21 +177,21 @@ string changes_change (void * webserver_request)
     a_node.text().set(summary.c_str());
     td_node = tr_node.append_child("td");
     if (subscription) {
-      xml_node a_node = td_node.append_child("a");
-      a_node.append_attribute("href") = string("unsubscribe" + convert_to_string (note)).c_str();
-      a_node.text().set(string("[" + translate("unsubscribe") + "]").c_str());
+      xml_node a_node2 = td_node.append_child("a");
+      a_node2.append_attribute("href") = string("unsubscribe" + convert_to_string (note)).c_str();
+      a_node2.text().set(string("[" + translate("unsubscribe") + "]").c_str());
     }
     td_node = tr_node.append_child("td");
     if (assignment) {
-      xml_node a_node = td_node.append_child("a");
-      a_node.append_attribute("href") = string("unassign" + convert_to_string (note)).c_str();
-      a_node.text().set(string("[" + translate("I have done my part on it") + "]").c_str());
+      xml_node a_node2 = td_node.append_child("a");
+      a_node2.append_attribute("href") = string("unassign" + convert_to_string (note)).c_str();
+      a_node2.text().set(string("[" + translate("I have done my part on it") + "]").c_str());
     }
     td_node = tr_node.append_child("td");
     if (level >= Filter_Roles::manager ()) {
-      xml_node a_node = td_node.append_child("a");
-      a_node.append_attribute("href") = string("delete" + convert_to_string (note)).c_str();
-      a_node.text().set(string("[" + translate("mark for deletion") + "]").c_str());
+      xml_node a_node2 = td_node.append_child("a");
+      a_node2.append_attribute("href") = string("delete" + convert_to_string (note)).c_str();
+      a_node2.text().set(string("[" + translate("mark for deletion") + "]").c_str());
     }
   }
   stringstream notesblock;
