@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,10 @@
 #include <navigation/passage.h>
 #include <changes/logic.h>
 #include <menu/logic.h>
+#include <pugixml/pugixml.hpp>
+
+
+using namespace pugi;
 
 
 string changes_changes_url ()
@@ -52,28 +56,28 @@ bool changes_changes_acl (void * webserver_request)
 
 string changes_changes (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   Database_Modifications database_modifications;
   
   
   // Handle AJAX call to load the summary of a change notification.
   if (request->query.count ("load")) {
     int identifier = convert_to_int (request->query["load"]);
-    string block;
+    stringstream block;
     Passage passage = database_modifications.getNotificationPassage (identifier);
     string link = filter_passage_link_for_opening_editor_at (passage.book, passage.chapter, passage.verse);
     string category = database_modifications.getNotificationCategory (identifier);
     if (category == changes_personal_category ()) category = emoji_smiling_face_with_smiling_eyes ();
     if (category == changes_bible_category ()) category = emoji_open_book ();
     string modification = database_modifications.getNotificationModification (identifier);
-    block.append ("<div id=\"entry" + convert_to_string (identifier) + "\">\n");
-    block.append ("<a href=\"expand\">" + emoji_file_folder () + "</a>\n");
-    block.append ("<a href=\"remove\">" + emoji_wastebasket () + "</a>\n");
-    block.append (link + "\n");
-    block.append (category + "\n");
-    block.append (modification + "\n");
-    block.append ("</div>\n");
-    return block;
+    block << "<div id=" << quoted("entry" + convert_to_string (identifier)) << + ">\n";
+    block << "<a href=" << quoted ("expand") << ">" << emoji_file_folder () << "</a>\n";
+    block << "<a href=" << quoted("remove") << ">" << emoji_wastebasket () << "</a>\n";
+    block << link << "\n";
+    block << category << "\n";
+    block << modification << "\n";
+    block << "</div>\n";
+    return block.str();
   }
   
   
@@ -97,7 +101,7 @@ string changes_changes (void * webserver_request)
     Passage passage = database_modifications.getNotificationPassage (id);
     if (passage.book) {
       Ipc_Focus::set (request, passage.book, passage.chapter, convert_to_int (passage.verse));
-      Navigation_Passage::recordHistory (request, passage.book, passage.chapter, convert_to_int (passage.verse));
+      Navigation_Passage::record_history (request, passage.book, passage.chapter, convert_to_int (passage.verse));
     }
     // Set the correct default Bible for the user.
     string bible = database_modifications.getNotificationBible (id);
@@ -207,8 +211,9 @@ string changes_changes (void * webserver_request)
   view.set_variable ("pendingidentifiers", pendingidentifiers);
   
   
-  string loading = "\"" + translate("Loading ...") + "\"";
-  string script = "var loading = " + loading + ";";
+  stringstream loading;
+  loading << quoted(translate("Loading ..."));
+  string script = "var loading = " + loading.str() + ";";
   config_logic_swipe_enabled (webserver_request, script);
   view.set_variable ("script", script);
 
@@ -232,7 +237,7 @@ string changes_changes (void * webserver_request)
       if (selectedbible == bible) cssclass = "active";
       string name (bible);
       if (name.empty ()) name = translate ("All Bibles");
-      view.add_iteration ("bibleselector", { make_pair ("selectbible", bible), make_pair ("biblename", name), make_pair ("class", cssclass) } );
+      view.add_iteration ("bibleselector", { pair ("selectbible", bible), pair ("biblename", name), pair ("class", cssclass) } );
     }
   }
 
@@ -252,7 +257,6 @@ string changes_changes (void * webserver_request)
   
   
   // Add links to clear the notifications from the individual contributors.
-  string dismissblock;
   vector <string> categories = database_modifications.getCategories ();
   for (auto & category : categories) {
     if (category == changes_personal_category ()) continue;
@@ -260,31 +264,26 @@ string changes_changes (void * webserver_request)
     string user = category;
     vector <int> ids = database_modifications.getNotificationTeamIdentifiers (username, user, selectedbible);
     if (!ids.empty ()) {
-      dismissblock.append ("<p>* <a href=\"?dismiss=");
-      dismissblock.append (user);
-      dismissblock.append ("&selectbible=");
-      dismissblock.append (selectedbible);
-      dismissblock.append ("\">");
-      dismissblock.append (user);
-      dismissblock.append (": ");
-      dismissblock.append (translate("all of them"));
-      dismissblock.append (": ");
-      dismissblock.append (convert_to_string (ids.size ()));
-      dismissblock.append ("</a></p>\n");
+      view.add_iteration ("individual", {
+        pair ("user", user),
+        pair ("selectedbible", selectedbible),
+        pair ("count", convert_to_string(ids.size()))
+      } );
     }
   }
-  view.set_variable ("dismissblock", dismissblock);
 
   
   // Add links to clear matching notifications of the various users.
   for (auto & category : categories) {
     if (category == changes_bible_category ()) continue;
     string user = category;
-    vector <int> personal_ids = database_modifications.getNotificationTeamIdentifiers (username, user, selectedbible);
-    string icon = category;
-    if (category == changes_personal_category ()) icon = emoji_smiling_face_with_smiling_eyes ();
-    if (!personal_ids.empty () && !bible_ids.empty ()) {
-      view.add_iteration ("matching", { make_pair ("user", user), make_pair ("icon", icon) } );
+    vector <int> personal_ids2 = database_modifications.getNotificationTeamIdentifiers (username, user, selectedbible);
+    string user_and_icon = translate ("user") + " " + category;
+    if (category == changes_personal_category ()) {
+      user_and_icon = translate ("me") + " " + emoji_smiling_face_with_smiling_eyes ();
+    }
+    if (!personal_ids2.empty () && !bible_ids.empty ()) {
+      view.add_iteration ("matching", { pair ("user", user), pair ("icon", user_and_icon) } );
     }
   }
   
