@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -45,10 +45,9 @@ string checks_settings_url ()
 }
 
 
-bool checks_settings_acl (void * webserver_request)
+bool checks_settings_acl ([[maybe_unused]] void * webserver_request)
 {
 #ifdef HAVE_CLIENT
-  (void) webserver_request;
   return true;
 #else
   return Filter_Roles::access_control (webserver_request, Filter_Roles::manager ());
@@ -58,7 +57,7 @@ bool checks_settings_acl (void * webserver_request)
 
 string checks_settings (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   
   
   string page;
@@ -70,11 +69,11 @@ string checks_settings (void * webserver_request)
   
   if (request->query.count ("bible")) {
     string bible = request->query["bible"];
-    if (bible == "") {
-      Dialog_List dialog_list = Dialog_List ("settings", translate("Select which Bible to manage"), "", "");
-      vector <string> bibles = access_bible_bibles (webserver_request);
-      for (auto bible : bibles) {
-        dialog_list.add_row (bible, "bible", bible);
+    if (bible.empty()) {
+      Dialog_List dialog_list = Dialog_List ("settings", translate("Select which Bible to manage"), string(), string());
+      vector <string> bibles = AccessBible::Bibles (webserver_request);
+      for (auto selectable_bible : bibles) {
+        dialog_list.add_row (selectable_bible, "bible", selectable_bible);
       }
       page += dialog_list.run ();
       return page;
@@ -82,7 +81,7 @@ string checks_settings (void * webserver_request)
       request->database_config_user()->setBible (bible);
     }
   }
-  string bible = access_bible_clamp (webserver_request, request->database_config_user()->getBible ());
+  string bible = AccessBible::Clamp (webserver_request, request->database_config_user()->getBible ());
 
   
   if (request->query.count ("run")) {
@@ -112,7 +111,13 @@ string checks_settings (void * webserver_request)
     Database_Config_Bible::setCheckSpaceBeforePunctuation (bible, checked);
   }
   view.set_variable ("spacebeforepunctuation", get_checkbox_status (Database_Config_Bible::getCheckSpaceBeforePunctuation (bible)));
-                      
+
+  
+  if (checkbox == "spacebeforefinalnotemarkup") {
+    Database_Config_Bible::setCheckSpaceBeforeFinalNoteMarker (bible, checked);
+  }
+  view.set_variable ("spacebeforefinalnotemarkup", get_checkbox_status (Database_Config_Bible::getCheckSpaceBeforeFinalNoteMarker (bible)));
+
                       
   if (checkbox == "sentencestructure") {
     Database_Config_Bible::setCheckSentenceStructure (bible, checked);
@@ -180,6 +185,18 @@ string checks_settings (void * webserver_request)
   view.set_variable ("frenchcitation", get_checkbox_status (Database_Config_Bible::getCheckFrenchCitationStyle (bible)));
 
   
+  if (checkbox == "transposenotes") {
+    Database_Config_Bible::setTransposeFixSpacesNotes (bible, checked);
+  }
+  view.set_variable ("transposenotes", get_checkbox_status (Database_Config_Bible::getTransposeFixSpacesNotes (bible)));
+
+  
+  if (checkbox == "validutf8") {
+    Database_Config_Bible::setCheckValidUTF8Text (bible, checked);
+  }
+  view.set_variable ("validutf8", get_checkbox_status (Database_Config_Bible::getCheckValidUTF8Text (bible)));
+
+  
   view.set_variable ("bible", bible);
 
   
