@@ -62,7 +62,7 @@ string Database_Bibles::chapterFolder (string bible, int book, int chapter)
 // It mimics Bibledit's native filesystem database directory structure.
 string Database_Bibles::icfURL ()
 {
-  return "https://alkitabkita.info/pengolah-usfm-tsi-untuk-bibledit/usfm-hasil/";
+  return "https://raw.githubusercontent.com/aranggitoar/pengolah-usfm-tsi-untuk-bibledit/main/usfm-hasil/";
 }
 
 
@@ -215,7 +215,7 @@ vector <int> Database_Bibles::getBooks (string bible)
   for (auto & book : books) {
     order.push_back (Database_Books::getOrderFromId (book));
   }
-  quick_sort (order, books, 0, static_cast<int>(order.size()));
+  quick_sort (order, books, 0, static_cast<unsigned>(order.size()));
 
   // Result.
   return books;
@@ -304,7 +304,7 @@ int Database_Bibles::getChapterAge (string bible, int book, int chapter)
     string file = files [files.size() - 1];
     string path = filter_url_create_path ({folder, file});
     int time = filter_url_file_modification_time (path);
-    int now = filter_date_seconds_since_epoch ();
+    int now = filter::date::seconds_since_epoch ();
     return now - time;
   }
   return 100'000'000;
