@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -128,9 +128,9 @@ void Database_Check::recordOutput (string bible, int book, int chapter, int vers
     sql.add ("AND data = ");
     sql.add (data);
     sql.add (";");
-    vector <string> result = sql.query () ["count(*)"];
-    if (!result.empty ()) {
-      count = convert_to_int (result [0]);
+    vector <string> count_result = sql.query () ["count(*)"];
+    if (!count_result.empty ()) {
+      count = convert_to_int (count_result [0]);
     }
     // Record the data no more than so often.
     if (count < 10) {
@@ -235,8 +235,8 @@ Passage Database_Check::getPassage (int id)
   vector <string> books = result ["book"];
   vector <string> chapters = result ["chapter"];
   vector <string> verses = result ["verse"];
-  for (unsigned int i = 0; i < books.size(); i++) {
-    Passage passage = Passage ("", convert_to_int (books[i]), convert_to_int (chapters[i]), verses[i]);
+  if (!books.empty()) {
+    Passage passage = Passage ("", convert_to_int (books[0]), convert_to_int (chapters[0]), verses[0]);
     return passage;
   }
   return Passage ("", 0, 0, "");
