@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <styles/logic.h>
 #include <filter/date.h>
 #include <database/logic.h>
+#include <database/config/general.h>
+#include <locale/logic.h>
 
 
 Database_Config_User::Database_Config_User (void * webserver_request_in)
@@ -48,13 +50,13 @@ map <string, string> database_config_user_cache;
 
 string Database_Config_User::file (string user)
 {
-  return filter_url_create_root_path (database_logic_databases (), "config", "user", user);
+  return filter_url_create_root_path ({database_logic_databases (), "config", "user", user});
 }
 
 
 string Database_Config_User::file (string user, const char * key)
 {
-  return filter_url_create_path (file (user), key);
+  return filter_url_create_path ({file (user), key});
 }
 
 
@@ -67,7 +69,7 @@ string Database_Config_User::mapkey (string user, const char * key)
 
 string Database_Config_User::getValue (const char * key, const char * default_value)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   string user = request->session_logic ()->currentUser ();
   return getValueForUser (user, key, default_value);
 }
@@ -122,7 +124,8 @@ int Database_Config_User::getIValueForUser (string user, const char * key, int d
 
 void Database_Config_User::setValue (const char * key, string value)
 {
-  string user = ((Webserver_Request *) webserver_request)->session_logic ()->currentUser ();
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+  string user = request->session_logic ()->currentUser ();
   setValueForUser (user, key, value);
 }
 
@@ -159,7 +162,8 @@ void Database_Config_User::setBValueForUser (string user, const char * key, bool
 
 vector <string> Database_Config_User::getList (const char * key)
 {
-  string user = ((Webserver_Request *) webserver_request)->session_logic ()->currentUser ();
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+  string user = request->session_logic ()->currentUser ();
   return getListForUser (user, key);
 }
 
@@ -188,7 +192,8 @@ vector <string> Database_Config_User::getListForUser (string user, const char *
 
 void Database_Config_User::setList (const char * key, vector <string> values)
 {
-  string user = ((Webserver_Request *) webserver_request)->session_logic ()->currentUser ();
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+  string user = request->session_logic ()->currentUser ();
   setListForUser (user, key, values);
 }
 
@@ -233,9 +238,9 @@ void Database_Config_User::trim ()
   // Reset the sprint month and year after some time.
   // When a user visits the Sprint page after a few days, it will then display the current Sprint.
   // If the Sprint is not reset, the user may enter new tasks in the wrong sprint.
-  int time = filter_date_seconds_since_epoch () - (2 * 24 * 3600);
+  int time = filter::date::seconds_since_epoch () - (2 * 24 * 3600);
   Database_Users database_users;
-  vector <string> users = database_users.getUsers ();
+  vector <string> users = database_users.get_users ();
   for (unsigned int i = 0; i < users.size(); i++) {
     string filename = file (users[i], keySprintMonth ());
     if (file_or_dir_exists (filename)) {
@@ -277,7 +282,7 @@ string Database_Config_User::getBible ()
 {
   string bible = getValue ("bible", "");
   // If the Bible does not exist, take the first one available.
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   Database_Bibles * database_bibles = request->database_bibles ();
   vector <string> bibles = database_bibles->getBibles ();
   if (find (bibles.begin (), bibles.end (), bible) == bibles.end ()) {
@@ -544,7 +549,8 @@ bool Database_Config_User::defaultBibleChecksNotification ()
 #ifdef HAVE_CLIENT
   return false;
 #else
-  int level = ((Webserver_Request *) webserver_request)->session_logic ()->currentLevel ();
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+  int level = request->session_logic ()->currentLevel ();
   return (level >= Filter_Roles::translator () && level <= Filter_Roles::manager ());
 #endif
 }
@@ -648,11 +654,21 @@ void Database_Config_User::setSuppressMailFromYourUpdatesNotes (bool value)
 
 vector <string> Database_Config_User::getActiveResources ()
 {
+  if (config_logic_indonesian_cloud_free ()) {
+    // In the Indonesian Cloud free, there's one central location for storing the active resources.
+    return Database_Config_General::getActiveResources ();
+  }
+  // Default values.
   return getList ("active-resources");
 }
 void Database_Config_User::setActiveResources (vector <string> values)
 {
-  setList ("active-resources", values);
+  if (config_logic_default_bibledit_configuration ()) {
+    setList ("active-resources", values);
+  }
+  if (config_logic_indonesian_cloud_free ()) {
+    Database_Config_General::setActiveResources (values);
+  }
 }
 
 
@@ -672,7 +688,7 @@ const char * Database_Config_User::keySprintMonth ()
 }
 int Database_Config_User::getSprintMonth ()
 {
-  return getIValue (keySprintMonth (), filter_date_numerical_month (filter_date_seconds_since_epoch ()));
+  return getIValue (keySprintMonth (), filter::date::numerical_month (filter::date::seconds_since_epoch ()));
 }
 void Database_Config_User::setSprintMonth (int value)
 {
@@ -686,7 +702,7 @@ const char * Database_Config_User::keySprintYear ()
 }
 int Database_Config_User::getSprintYear ()
 {
-  return getIValue (keySprintYear (), filter_date_numerical_year (filter_date_seconds_since_epoch ()));
+  return getIValue (keySprintYear (), filter::date::numerical_year (filter::date::seconds_since_epoch ()));
 }
 void Database_Config_User::setSprintYear (int value)
 {
@@ -994,23 +1010,50 @@ void Database_Config_User::setSyncKey (string key)
 }
 
 
+
+//const char * site_language_key ()
+//{
+//  return "site-language";
+//}
+//string Database_Config_User::getSiteLanguage ()
+//{
+//  // The default value is "default".
+//  // That means: Take the system setting. The user has no language preference.
+//  return getValue (site_language_key (), "");
+//}
+//void Database_Config_User::setSiteLanguage (string value)
+//{
+//  setValue (site_language_key (), value);
+//}
+
+
+const char * general_font_size_key ()
+{
+  return "general-font-size";
+}
 int Database_Config_User::getGeneralFontSize ()
 {
-  return getIValue ("general-font-size", 100);
+  // Default value, see https://github.com/bibledit/cloud/issues/509
+  return getIValue (general_font_size_key (), 112);
 }
 void Database_Config_User::setGeneralFontSize (int size)
 {
-  setIValue ("general-font-size", size);
+  setIValue (general_font_size_key (), size);
 }
 
 
+const char * menu_font_size_key ()
+{
+  return "menu-font-size";
+}
 int Database_Config_User::getMenuFontSize ()
 {
-  return getIValue ("menu-font-size", 100);
+  // Default value, see https://github.com/bibledit/cloud/issues/509
+  return getIValue (menu_font_size_key (), 112);
 }
 void Database_Config_User::setMenuFontSize (int size)
 {
-  setIValue ("menu-font-size", size);
+  setIValue (menu_font_size_key (), size);
 }
 
 
@@ -1054,13 +1097,35 @@ void Database_Config_User::setGreekFontSize (int size)
 }
 
 
+const char * vertical_caret_position_key ()
+{
+  return "vertical-caret-position";
+}
 int Database_Config_User::getVerticalCaretPosition ()
 {
-  return getIValue ("vertical-caret-position", 50);
+  // Updated default value, see https://github.com/bibledit/cloud/issues/509
+  return getIValue (vertical_caret_position_key (), 30);
 }
 void Database_Config_User::setVerticalCaretPosition (int position)
 {
-  setIValue ("vertical-caret-position", position);
+  setIValue (vertical_caret_position_key (), position);
+}
+
+
+const char * current_theme_style_key ()
+{
+  return "current-theme-style";
+}
+int Database_Config_User::getCurrentTheme ()
+{
+  // Indonesian Cloud Free
+  // Set the "Light" theme as the default theme.
+  if (config_logic_indonesian_cloud_free ()) return getIValue (current_theme_style_key (), 1);
+  return getIValue (current_theme_style_key (), 0);
+}
+void Database_Config_User::setCurrentTheme (int index)
+{
+  setIValue (current_theme_style_key (), index);
 }
 
 
@@ -1084,23 +1149,31 @@ void Database_Config_User::setWorkspaceMenuFadeoutDelay (int value)
 }
 
 
+const char * editing_allowed_difference_chapter_key ()
+{
+  return "editing-allowed-difference-chapter";
+}
 int Database_Config_User::getEditingAllowedDifferenceChapter ()
 {
-  return getIValue ("editing-allowed-difference-chapter", 20);
+  return getIValue (editing_allowed_difference_chapter_key (), 20);
 }
 void Database_Config_User::setEditingAllowedDifferenceChapter (int value)
 {
-  setIValue ("editing-allowed-difference-chapter", value);
+  setIValue (editing_allowed_difference_chapter_key (), value);
 }
 
 
+const char * editing_allowed_difference_verse_key ()
+{
+  return "editing-allowed-difference-verse";
+}
 int Database_Config_User::getEditingAllowedDifferenceVerse ()
 {
-  return getIValue ("editing-allowed-difference-verse", 75);
+  return getIValue (editing_allowed_difference_verse_key (), 75);
 }
 void Database_Config_User::setEditingAllowedDifferenceVerse (int value)
 {
-  setIValue ("editing-allowed-difference-verse", value);
+  setIValue (editing_allowed_difference_verse_key (), value);
 }
 
 
@@ -1114,7 +1187,7 @@ bool Database_Config_User::getBasicInterfaceModeDefault ()
   return true;
 #endif
   // The app running on a workspace or laptop have default to basic mode for a lower role.
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   int level = request->session_logic ()->currentLevel ();
   if (level <= Filter_Roles::manager ()) return true;
   // Higher role: default to advanced mode.
@@ -1172,14 +1245,18 @@ void Database_Config_User::setIncludeRelatedPassages (bool value)
 }
 
 
+const char * enabled_visual_editors_key ()
+{
+  return "enabled-visual-editors";
+}
 int Database_Config_User::getFastSwitchVisualEditors ()
 {
-  // Initially only the visual verse editor is enabled.
-  return getIValue ("enabled-visual-editors", 2);
+  // Updated default values, see https://github.com/bibledit/cloud/issues/509
+  return getIValue (enabled_visual_editors_key (), 0);
 }
 void Database_Config_User::setFastSwitchVisualEditors (int value)
 {
-  setIValue ("enabled-visual-editors", value);
+  setIValue (enabled_visual_editors_key (), value);
 }
 
 
@@ -1360,3 +1437,63 @@ void Database_Config_User::setReceiveFocusedReferenceFromParatext (bool value)
 {
   setBValue (receive_focused_reference_from_paratext_key (), value);
 }
+
+
+const char * receive_focused_reference_from_accordance_key ()
+{
+  return "receive-focused-reference-from-accordance";
+}
+bool Database_Config_User::getReceiveFocusedReferenceFromAccordance ()
+{
+  return getBValue (receive_focused_reference_from_accordance_key (), true);
+}
+void Database_Config_User::setReceiveFocusedReferenceFromAccordance (bool value)
+{
+  setBValue (receive_focused_reference_from_accordance_key (), value);
+}
+
+
+const char * use_colored_note_status_labels_key ()
+{
+  return "use-colored-note-status-labels";
+}
+bool Database_Config_User::getUseColoredNoteStatusLabels ()
+{
+  return getBValue (use_colored_note_status_labels_key (), false);
+}
+void Database_Config_User::setUseColoredNoteStatusLabels (bool value)
+{
+  setBValue (use_colored_note_status_labels_key (), value);
+}
+
+
+const char * notes_date_format_key ()
+{
+  return "notes_date-format";
+}
+int Database_Config_User::getNotesDateFormat ()
+{
+  return getIValue (notes_date_format_key (), 0);
+}
+void Database_Config_User::setNotesDateFormat (int value)
+{
+  setIValue (notes_date_format_key (), value);
+}
+
+
+const char * change_notifications_bibles_key ()
+{
+  return "change-notifications-bibles";
+}
+vector <string> Database_Config_User::getChangeNotificationsBibles ()
+{
+  return getList (change_notifications_bibles_key ());
+}
+vector <string> Database_Config_User::getChangeNotificationsBiblesForUser (const string & user)
+{
+  return getListForUser (user, change_notifications_bibles_key ());
+}
+void Database_Config_User::setChangeNotificationsBibles (const vector <string>& values)
+{
+  setList (change_notifications_bibles_key (), values);
+}
