@@ -33,6 +33,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <webserver/request.h>
 #include <jsonxx/jsonxx.h>
 #include <database/logic.h>
+#include <time.h>
 
 
 using namespace jsonxx;
@@ -192,19 +193,19 @@ void Database_Notes::trim ()
   vector <string> bits1 = filter_url_scandir (main_folder);
   for (auto bit1 : bits1) {
     if (bit1.length () == 3) {
-      string folder = filter_url_create_path ({main_folder, bit1});
-      vector <string> bits2 = filter_url_scandir (folder);
+      string folder1 = filter_url_create_path ({main_folder, bit1});
+      vector <string> bits2 = filter_url_scandir (folder1);
       if (bits2.empty ()) {
-        Database_Logs::log (message + folder);
-        remove (folder.c_str ());
+        Database_Logs::log (message + folder1);
+        remove (folder1.c_str ());
       }
       for (auto bit2 : bits2) {
         if (bit2.length () == 3) {
-          string folder = filter_url_create_path ({main_folder, bit1, bit2});
-          vector <string> bits3 = filter_url_scandir (folder);
+          string folder2 = filter_url_create_path ({main_folder, bit1, bit2});
+          vector <string> bits3 = filter_url_scandir (folder2);
           if (bits3.empty ()) {
-            Database_Logs::log (message + folder);
-            remove (folder.c_str());
+            Database_Logs::log (message + folder2);
+            remove (folder2.c_str());
           }
         }
       }
@@ -501,11 +502,9 @@ vector <int> Database_Notes::get_identifiers ()
 string Database_Notes::assemble_contents (int identifier, string contents)
 {
   string new_contents = get_contents (identifier);
-  int time = filter_date_seconds_since_epoch ();
-  string datetime = convert_to_string (filter_date_numerical_month_day (time)) + "/" + convert_to_string (filter_date_numerical_month (time)) + "/" + convert_to_string (filter_date_numerical_year (time));
+  string datetime = filter::date::localized_date_format (webserver_request);
   Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   string user = request->session_logic ()->currentUser ();
-
   // To make the notes more readable, add whitespace between the comments.
   bool is_initial_comment = new_contents.empty ();
   if (!is_initial_comment) {
@@ -665,19 +664,19 @@ vector <int> Database_Notes::select_notes (vector <string> bibles, int book, int
       break;
     case 1:
       // Select notes that have been edited during the last 30 days.
-      time = filter_date_seconds_since_epoch () - 30 * 24 * 3600;
+      time = filter::date::seconds_since_epoch () - 30 * 24 * 3600;
       break;
     case 2:
       // Select notes that have been edited during the last 7 days.
-      time = filter_date_seconds_since_epoch () - 7 * 24 * 3600;
+      time = filter::date::seconds_since_epoch () - 7 * 24 * 3600;
       break;
     case 3:
       // Select notes that have been edited since yesterday.
-      time = filter_date_seconds_since_epoch () - 1 * 24 * 3600 - filter_date_numerical_hour (filter_date_seconds_since_epoch ()) * 3600;
+      time = filter::date::seconds_since_epoch () - 1 * 24 * 3600 - filter::date::numerical_hour (filter::date::seconds_since_epoch ()) * 3600;
       break;
     case 4:
       // Select notes that have been edited today.
-      time = filter_date_seconds_since_epoch () - filter_date_numerical_hour (filter_date_seconds_since_epoch ()) * 3600;
+      time = filter::date::seconds_since_epoch () - filter::date::numerical_hour (filter::date::seconds_since_epoch ()) * 3600;
       break;
   }
   if (time != 0) {
@@ -694,23 +693,23 @@ vector <int> Database_Notes::select_notes (vector <string> bibles, int book, int
       break;
     case 1:
       // Select notes that have not been edited for a day.
-      nonedit = filter_date_seconds_since_epoch () - 1 * 24 * 3600;
+      nonedit = filter::date::seconds_since_epoch () - 1 * 24 * 3600;
       break;
     case 2:
       // Select notes that have not been edited for two days.
-      nonedit = filter_date_seconds_since_epoch () - 2 * 24 * 3600;
+      nonedit = filter::date::seconds_since_epoch () - 2 * 24 * 3600;
       break;
     case 3:
       // Select notes that have not been edited for a week.
-      nonedit = filter_date_seconds_since_epoch () - 7 * 24 * 3600;
+      nonedit = filter::date::seconds_since_epoch () - 7 * 24 * 3600;
       break;
     case 4:
       // Select notes that have not been edited for a month.
-      nonedit = filter_date_seconds_since_epoch () - 30 * 24 * 3600;
+      nonedit = filter::date::seconds_since_epoch () - 30 * 24 * 3600;
       break;
     case 5:
       // Select notes that have not been edited for a year.
-      nonedit = filter_date_seconds_since_epoch () - 365 * 24 * 3600;
+      nonedit = filter::date::seconds_since_epoch () - 365 * 24 * 3600;
       break;
   }
   if (nonedit != 0) {
@@ -1386,7 +1385,7 @@ string Database_Notes::get_severity (int identifier)
   int severity = get_raw_severity (identifier);
   vector <string> standard = standard_severities ();
   string severitystring;
-  if ((severity >= 0) && (severity < (int)standard.size())) severitystring = standard [severity];
+  if ((severity >= 0) && (severity < (int)standard.size())) severitystring = standard [static_cast<size_t> (severity)];
   if (severitystring.empty()) severitystring = "Normal";
   severitystring = translate (severitystring.c_str());
   return severitystring;
@@ -1474,7 +1473,7 @@ void Database_Notes::set_public (int identifier, bool value)
 void Database_Notes::note_modified_actions (int identifier)
 {
   // Update 'modified' field.
-  set_modified (identifier, filter_date_seconds_since_epoch());
+  set_modified (identifier, filter::date::seconds_since_epoch());
 }
 
 
@@ -1879,17 +1878,17 @@ vector <string> Database_Notes::set_bulk (string json)
   for (size_t i = 0; i < bulk.size (); i++) {
     
     // Get all the different fields for this note.
-    Object note = bulk.get<Object>((int)i);
+    Object note = bulk.get<Object>(static_cast<unsigned>(i));
     string assigned = note.get<String> ("a");
     string bible = note.get<String> ("b");
     string contents = note.get<String> ("c");
-    int identifier = note.get<Number> ("i");
-    int modified = note.get<Number> ("m");
+    int identifier = static_cast<int>(note.get<Number> ("i"));
+    int modified = static_cast<int>(note.get<Number> ("m"));
     string passage = note.get<String> ("p");
     string subscriptions = note.get<String> ("sb");
     string summary = note.get<String> ("sm");
     string status = note.get<String> ("st");
-    int severity = note.get<Number> ("sv");
+    int severity = static_cast<int>(note.get<Number> ("sv"));
     
     // Feedback about which note it received in bulk.
     summaries.push_back (summary);
@@ -1908,8 +1907,8 @@ vector <string> Database_Notes::set_bulk (string json)
     note2 << summary_key () << summary;
     note2 << status_key () << status;
     note2 << severity_key () << convert_to_string (severity);
-    string json = note2.json ();
-    filter_url_file_put_contents (path, json);
+    string json2 = note2.json ();
+    filter_url_file_put_contents (path, json2);
     
     // Update the indexes.
     update_database (identifier);
