@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -51,15 +51,14 @@ string edit_save_url ()
 bool edit_save_acl (void * webserver_request)
 {
   if (Filter_Roles::access_control (webserver_request, Filter_Roles::translator ())) return true;
-  bool read, write;
-  access_a_bible (webserver_request, read, write);
+  auto [ read, write ] = AccessBible::Any (webserver_request);
   return read;
 }
 
 
 string edit_save (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   bool post_complete = (request->post.count ("bible") && request->post.count ("book") && request->post.count ("chapter") && request->post.count ("html") && request->post.count ("checksum"));
   if (!post_complete) {
@@ -91,7 +90,7 @@ string edit_save (void * webserver_request)
     return translate("Save failure");
   }
   
-  if (!access_bible_book_write (request, "", bible, book)) {
+  if (!AccessBible::BookWrite (request, string(), bible, book)) {
     return translate("No write access");
   }
 
@@ -102,8 +101,8 @@ string edit_save (void * webserver_request)
   editor_export.stylesheet (stylesheet);
   editor_export.run ();
   string user_usfm = editor_export.get ();
-
-  string ancestor_usfm = getLoadedUsfm (webserver_request, bible, book, chapter, unique_id);
+  
+  string ancestor_usfm = getLoadedUsfm2 (webserver_request, bible, book, chapter, unique_id);
   
   vector <BookChapterData> book_chapter_text = usfm_import (user_usfm, stylesheet);
   if (book_chapter_text.size () != 1) {
@@ -122,7 +121,7 @@ string edit_save (void * webserver_request)
   // Collect some data about the changes for this user
   // and for a possible merge of the user's data with the server's data.
   string username = request->session_logic()->currentUser ();
-  int oldID = request->database_bibles()->getChapterId (bible, book, chapter);
+  [[maybe_unused]] int oldID = request->database_bibles()->getChapterId (bible, book, chapter);
   string server_usfm = request->database_bibles()->getChapter (bible, book, chapter);
   string newText = user_usfm;
   string oldText = ancestor_usfm;
@@ -141,6 +140,7 @@ string edit_save (void * webserver_request)
   }
   
   // Check on the merge.
+  filter_merge_add_book_chapter (conflicts, book, chapter);
   bible_logic_merge_irregularity_mail ({username}, conflicts);
   
   // Check whether the USFM on disk has changed compared to the USFM that was loaded in the editor.
@@ -152,18 +152,19 @@ string edit_save (void * webserver_request)
   // Because the user's editor may not yet have loaded this updated Bible text.
   // https://github.com/bibledit/cloud/issues/340
   if (ancestor_usfm != server_usfm) {
-    bible_logic_recent_save_email (bible, book, chapter, 0, username, ancestor_usfm, server_usfm);
+    bible_logic_recent_save_email (bible, book, chapter, username, ancestor_usfm, server_usfm);
   }
 
   // Safely store the chapter.
   string explanation;
   string message = usfm_safely_store_chapter (request, bible, book, chapter, user_usfm, explanation);
-  bible_logic_unsafe_save_mail (message, explanation, username, user_usfm);
-  
+  bible_logic_unsafe_save_mail (message, explanation, username, user_usfm, book, chapter);
+
+  // If an error message was given, then return that message to the browser.
   if (!message.empty ()) return message;
 
   // In server configuration, store details for the user's changes.
-#ifndef HAVE_CLIENT
+#ifdef HAVE_CLOUD
   int newID = request->database_bibles()->getChapterId (bible, book, chapter);
   Database_Modifications database_modifications;
   database_modifications.recordUserSave (username, bible, book, chapter, oldID, oldText, newID, newText);
@@ -171,13 +172,11 @@ string edit_save (void * webserver_request)
     Database_Git::store_chapter (username, bible, book, chapter, oldText, newText);
   }
   rss_logic_schedule_update (username, bible, book, chapter, oldText, newText);
-#else
-  (void) oldID;
 #endif
 
   // Store a copy of the USFM loaded in the editor for later reference.
-  storeLoadedUsfm (webserver_request, bible, book, chapter, unique_id);
-  
+  storeLoadedUsfm2 (webserver_request, bible, book, chapter, unique_id);
+
   // Convert the stored USFM to html.
   // This converted html should be the same as the saved html.
   // If it differs, signal the browser to reload the chapter.
@@ -188,7 +187,8 @@ string edit_save (void * webserver_request)
   editor_usfm2html.run ();
   string converted_html = editor_usfm2html.get ();
   // Convert to XML for comparison.
-  // Remove spaces before comparing, so that entering a space in the editor does not cause a reload.
+  // Remove spaces before comparing.
+  // Goal: Entering a space in the editor does not cause a reload.
   html = html2xml (html);
   html = filter_string_str_replace (" ", "", html);
   html = filter_string_str_replace (unicode_non_breaking_space_entity (), "", html);
