@@ -118,8 +118,8 @@ void email_send ()
       // rather than trying to send them all, and have them all cause a 'login denied' error.
       if (login_denied) {
         vector <int> ids = database_mail.getAllMails ();
-        for (auto id : ids) {
-          database_mail.postpone (id);
+        for (auto id2 : ids) {
+          database_mail.postpone (id2);
         }
         Database_Logs::log ("Postponing sending " + convert_to_string (ids.size()) + " emails", Filter_Roles::manager ());
         break;
@@ -137,7 +137,7 @@ static vector <string> payload_text;
 
 
 struct upload_status {
-  int lines_read;
+  size_t lines_read;
 };
 
 
@@ -152,7 +152,7 @@ static size_t payload_source (void *ptr, size_t size, size_t nmemb, void *userp)
     return 0;
   }
   
-  if (upload_ctx->lines_read >= (int)payload_text.size()) {
+  if (upload_ctx->lines_read >= payload_text.size()) {
     return 0;
   }
 
@@ -224,10 +224,10 @@ string email_send ([[maybe_unused]] string to_mail,
 
   upload_ctx.lines_read = 0;
 
-  int seconds = filter_date_seconds_since_epoch ();
+  int seconds = filter::date::seconds_since_epoch ();
   payload_text.clear();
   string payload;
-  payload = "Date: " + convert_to_string (filter_date_numerical_year (seconds)) + "/" + convert_to_string (filter_date_numerical_month (seconds)) + "/" + convert_to_string (filter_date_numerical_month_day (seconds)) + " " + convert_to_string (filter_date_numerical_hour (seconds)) + ":" + convert_to_string (filter_date_numerical_minute (seconds)) + "\n";
+  payload = "Date: " + convert_to_string (filter::date::numerical_year (seconds)) + "/" + convert_to_string (filter::date::numerical_month (seconds)) + "/" + convert_to_string (filter::date::numerical_month_day (seconds)) + " " + convert_to_string (filter::date::numerical_hour (seconds)) + ":" + convert_to_string (filter::date::numerical_minute (seconds)) + "\n";
   payload_text.push_back (payload);
   payload = "To: <" + to_mail + "> " + to_name + "\n";
   payload_text.push_back (payload);
@@ -375,6 +375,10 @@ void email_schedule (string to, string subject, string body, int time)
 // If everything's OK, it returns nothing.
 string email_setup_information (bool require_send, bool require_receive)
 {
+#ifdef HAVE_CLIENT
+  (void) require_send;
+  (void) require_receive;
+#endif
 #ifdef HAVE_CLOUD
   bool incomplete = false;
   if (Database_Config_General::getSiteMailName ().empty ()) incomplete = true;
