@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -20,10 +20,14 @@
 #include <filter/date.h>
 #include <filter/string.h>
 #include <database/config/general.h>
+#include <webserver/request.h>
+
+
+namespace filter::date {
 
 
 // Gets the second within the minute from the seconds since the Unix epoch.
-int filter_date_numerical_second (int seconds)
+int numerical_second (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -33,7 +37,7 @@ int filter_date_numerical_second (int seconds)
 
 
 // Gets the minute within the hour from the seconds since the Unix epoch.
-int filter_date_numerical_minute (int seconds)
+int numerical_minute (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -43,7 +47,7 @@ int filter_date_numerical_minute (int seconds)
 
 
 // Gets the hour within the day from the seconds since the Unix epoch.
-int filter_date_numerical_hour (int seconds)
+int numerical_hour (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -53,7 +57,7 @@ int filter_date_numerical_hour (int seconds)
 
 
 // The numerical day of the month from 1 to 31.
-int filter_date_numerical_month_day (int seconds)
+int numerical_month_day (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -63,7 +67,7 @@ int filter_date_numerical_month_day (int seconds)
 
 
 // The numerical day of the week: 0 (for Sunday) through 6 (for Saturday)
-int filter_date_numerical_week_day (int seconds)
+int numerical_week_day (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -74,7 +78,7 @@ int filter_date_numerical_week_day (int seconds)
 
 // A C++ equivalent for PHP's date ("n") function.
 // Numeric representation of a month: 1 through 12.
-int filter_date_numerical_month (int seconds)
+int numerical_month (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -85,7 +89,7 @@ int filter_date_numerical_month (int seconds)
 
 // A C++ equivalent for PHP's date ("Y") function.
 // A full numeric representation of a year, 4 digits: 2014.
-int filter_date_numerical_year (int seconds)
+int numerical_year (int seconds)
 {
   time_t tt = seconds;
   tm utc_tm = *gmtime(&tt);
@@ -96,7 +100,7 @@ int filter_date_numerical_year (int seconds)
 
 
 // This function gives the number of microseconds within the current second.
-int filter_date_numerical_microseconds ()
+int numerical_microseconds ()
 {
   auto now = chrono::system_clock::now ();
   auto duration = now.time_since_epoch ();
@@ -107,7 +111,7 @@ int filter_date_numerical_microseconds ()
 
 
 // This function returns the seconds since the Unix epoch, which is 1 January 1970 UTC.
-int filter_date_seconds_since_epoch ()
+int seconds_since_epoch ()
 {
   auto now = chrono::system_clock::now ();
   auto duration = now.time_since_epoch ();
@@ -117,16 +121,16 @@ int filter_date_seconds_since_epoch ()
 
 
 // Returns the seconds since the Unix epoch for $year and $month and $day.
-int filter_date_seconds_since_epoch (int year, int month, int day)
+int seconds_since_epoch (int year, int month, int day)
 {
   int seconds = 0;
   bool done = false;
   bool hit = false;
   do {
     seconds += 86400;
-    int myyear = filter_date_numerical_year (seconds);
-    int mymonth = filter_date_numerical_month (seconds);
-    int myday = filter_date_numerical_month_day (seconds);
+    int myyear = numerical_year (seconds);
+    int mymonth = numerical_month (seconds);
+    int myday = numerical_month_day (seconds);
     if ((year == myyear) && (month == mymonth)) hit = true;
     done = ((year == myyear) && (month == mymonth) && (day == myday));
     if (hit) if (month != mymonth) done = true;
@@ -138,7 +142,7 @@ int filter_date_seconds_since_epoch (int year, int month, int day)
 // This function takes the "seconds" parameter,
 // corrects it according to the local timezone,
 // and returns it.
-int filter_date_local_seconds (int seconds)
+int local_seconds (int seconds)
 {
   int offset = Database_Config_General::getTimezone ();
   seconds += (offset * 3600);
@@ -146,7 +150,7 @@ int filter_date_local_seconds (int seconds)
 }
 
 
-bool filter_date_is_first_business_day_of_month (int monthday, int weekday)
+bool is_first_business_day_of_month (int monthday, int weekday)
 {
   if (monthday == 1) {
     if (weekday == 1) return true;
@@ -163,18 +167,18 @@ bool filter_date_is_first_business_day_of_month (int monthday, int weekday)
 }
 
 
-int filter_date_get_last_business_day_of_month (int year, int month)
+int get_last_business_day_of_month (int year, int month)
 {
   int myyear = year;
   int mymonth = month;
-  filter_date_get_next_month (mymonth, myyear);
-  int seconds = filter_date_seconds_since_epoch (myyear, mymonth, 1);
+  get_next_month (mymonth, myyear);
+  int seconds = seconds_since_epoch (myyear, mymonth, 1);
   int iterations = 0;
   do {
     seconds -= 86400;
-    int weekday = filter_date_numerical_week_day (seconds);
+    int weekday = numerical_week_day (seconds);
     if ((weekday == 1) || (weekday == 2) || (weekday == 3) || (weekday == 4) || (weekday == 5)) {
-      return filter_date_numerical_month_day (seconds);
+      return numerical_month_day (seconds);
     }
     iterations++;
   } while (iterations < 10);
@@ -182,10 +186,10 @@ int filter_date_get_last_business_day_of_month (int year, int month)
 }
 
 
-bool filter_date_is_business_day (int year, int month, int day)
+bool is_business_day (int year, int month, int day)
 {
-  int seconds = filter_date_seconds_since_epoch (year, month, day);
-  int weekday = filter_date_numerical_week_day (seconds);
+  int seconds = seconds_since_epoch (year, month, day);
+  int weekday = numerical_week_day (seconds);
   if ((weekday == 1) || (weekday == 2) || (weekday == 3) || (weekday == 4) || (weekday == 5)) {
     return true;
   }
@@ -193,7 +197,7 @@ bool filter_date_is_business_day (int year, int month, int day)
 }
 
 
-void filter_date_get_previous_month (int & month, int & year)
+void get_previous_month (int & month, int & year)
 {
   month--;
   if (month <= 0) {
@@ -203,7 +207,7 @@ void filter_date_get_previous_month (int & month, int & year)
 }
 
 
-void filter_date_get_next_month (int & month, int & year)
+void get_next_month (int & month, int & year)
 {
   month++;
   if (month > 12) {
@@ -213,7 +217,7 @@ void filter_date_get_next_month (int & month, int & year)
 }
 
 
-string filter_date_day_rfc822 (int day)
+string day_rfc822 (int day)
 {
   if (day == 0) return "Sun";
   if (day == 1) return "Mon";
@@ -226,7 +230,7 @@ string filter_date_day_rfc822 (int day)
 }
 
 
-string filter_date_month_rfc822 (int month)
+string month_rfc822 (int month)
 {
   if (month ==  1) return "Jan";
   if (month ==  2) return "Feb";
@@ -247,7 +251,7 @@ string filter_date_month_rfc822 (int month)
 // Converts the number of $seconds since the Unix epoch
 // to date and time values according to RFC 822,
 // e.g.: Wed, 02 Oct 2002 15:00:00 +0200.
-string filter_date_rfc822 (int seconds)
+string rfc822 (int seconds)
 {
   string rfc822;
   // The feed validator at https://validator.w3.org/feed/ says:
@@ -257,25 +261,25 @@ string filter_date_rfc822 (int seconds)
   // It continues to say that:
   // If it turns out that computing the correct day of week is impractical using the software you have available, then RFC 822 permits omitting both the day of the week and the subsequent comma from the value.
   // So to work around the error in the validator, the day of the week is left out.
-  // int weekday = filter_date_numerical_week_day (seconds);
-  // rfc822.append (filter_date_day_rfc822 (weekday));
+  // int weekday = numerical_week_day (seconds);
+  // rfc822.append (day_rfc822 (weekday));
   // rfc822.append (", ");
-  string monthday = convert_to_string (filter_date_numerical_month_day (seconds));
+  string monthday = convert_to_string (numerical_month_day (seconds));
   rfc822.append (filter_string_fill (monthday, 2, '0'));
   rfc822.append (" ");
-  int month = filter_date_numerical_month (seconds);
-  rfc822.append (filter_date_month_rfc822 (month));
+  int month = numerical_month (seconds);
+  rfc822.append (month_rfc822 (month));
   rfc822.append (" ");
-  int year = filter_date_numerical_year (seconds);
+  int year = numerical_year (seconds);
   rfc822.append (convert_to_string (year));
   rfc822.append (" ");
-  string hour = convert_to_string (filter_date_numerical_hour (seconds));
+  string hour = convert_to_string (numerical_hour (seconds));
   rfc822.append (filter_string_fill (hour, 2, '0'));
   rfc822.append (":");
-  string minute = convert_to_string (filter_date_numerical_minute (seconds));
+  string minute = convert_to_string (numerical_minute (seconds));
   rfc822.append (filter_string_fill (minute, 2, '0'));
   rfc822.append (":");
-  string second = convert_to_string (filter_date_numerical_second (seconds));
+  string second = convert_to_string (numerical_second (seconds));
   rfc822.append (filter_string_fill (second, 2, '0'));
   rfc822.append (" ");
   int timezone = Database_Config_General::getTimezone ();
@@ -286,3 +290,72 @@ string filter_date_rfc822 (int seconds)
   rfc822.append ("00");
   return rfc822;
 }
+
+
+// Calculates the number of microseconds elapsed since $start.
+// It returns the elapsed number of microseconds.
+long elapsed_microseconds (long start)
+{
+  auto now = chrono::system_clock::now ();
+  auto duration = now.time_since_epoch ();
+  auto microseconds = chrono::duration_cast<std::chrono::microseconds>(duration).count();
+  long elapsed = microseconds - start;
+  return elapsed;
+}
+
+
+string localized_date_format ()
+{
+  time_t tt;
+  time (&tt);
+  struct tm * localtm = localtime (&tt);
+  char buffer[20];
+  strftime(buffer, sizeof(buffer), "%x", localtm);
+  return string (buffer);
+}
+
+
+string date_format_to_text (date_format format)
+{
+  switch (format) {
+    case dd_mm_yyyy: return "dd/mm/yyyy";
+    case mm_dd_yyyy: return "mm/dd/yyyy";
+    case yyyy_mn_dd: return "yyyy-mm-dd";
+  }
+  return string();
+}
+
+
+string localized_date_format (void * webserver_request)
+{
+  int time = seconds_since_epoch ();
+  
+  string day = convert_to_string (numerical_month_day (time));
+  string month = convert_to_string (numerical_month (time));
+  string year = convert_to_string (numerical_year (time));
+
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+  date_format df = static_cast<date_format>(request->database_config_user()->getNotesDateFormat());
+
+  switch (df) {
+    case dd_mm_yyyy:
+    {
+      return day + "/" + month + "/" + year;
+    }
+    case mm_dd_yyyy:
+    {
+      return month + "/" + day + "/" + year;
+    }
+    case yyyy_mn_dd:
+    default:
+    {
+      return year + "-" + month + "-" + day;
+    }
+  }
+  
+  return string();
+}
+
+
+}
+
