@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,8 +17,17 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 
+// Suppress errors in Visual Studio 2019.
+// No longer needed since upgrading the UTF8 library?
+#define _SILENCE_CXX17_ITERATOR_BASE_CLASS_DEPRECATION_WARNING 1
+#define _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING 1
+
 #include <filter/string.h>
+#pragma GCC diagnostic push
+#pragma clang diagnostic ignored "-Wimplicit-int-conversion"
+#pragma clang diagnostic ignored "-Wsign-conversion"
 #include <utf8/utf8.h>
+#pragma GCC diagnostic pop
 #include <filter/url.h>
 #include <filter/md5.h>
 #include <filter/date.h>
@@ -29,6 +38,27 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #ifdef HAVE_WINDOWS
 #include <codecvt>
 #endif
+#ifdef HAVE_ICU
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#pragma clang diagnostic ignored "-Wdocumentation"
+#pragma clang diagnostic ignored "-Wsign-conversion"
+#include <unicode/ustdio.h>
+#include <unicode/normlzr.h>
+#include <unicode/utypes.h>
+#include <unicode/unistr.h>
+#include <unicode/translit.h>
+#pragma clang diagnostic pop
+#endif
+#ifdef HAVE_CLOUD
+#include <libxml/tree.h>
+#include <libxml/HTMLparser.h>
+#endif
+
+
+#ifdef HAVE_ICU
+using namespace icu;
+#endif
 
 
 // A C++ equivalent for PHP's explode function.
@@ -214,6 +244,7 @@ float convert_to_float (string s)
 
 bool convert_to_bool (string s)
 {
+  if (s.empty()) return false;
   if (s == "true") return true;
   if (s == "TRUE") return true;
   bool b;
@@ -351,7 +382,7 @@ bool filter_string_is_numeric (string s)
 string escape_special_xml_characters (string s)
 {
   s = filter_string_str_replace ("&", "&amp;", s);
-  s = filter_string_str_replace ("\"", "&quot;", s);
+  s = filter_string_str_replace (R"(")", "&quot;", s);
   s = filter_string_str_replace ("'", "&apos;", s);
   s = filter_string_str_replace ("<", "&lt;", s);
   s = filter_string_str_replace (">", "&gt;", s);
@@ -362,7 +393,7 @@ string escape_special_xml_characters (string s)
 // This unescapes the five special XML characters.
 string unescape_special_xml_characters (string s)
 {
-  s = filter_string_str_replace ("&quot;", """", s);
+  s = filter_string_str_replace ("&quot;", R"(")", s);
   s = filter_string_str_replace ("&amp;", "&", s);
   s = filter_string_str_replace ("&apos;", "'", s);
   s = filter_string_str_replace ("&lt;", "<", s);
@@ -376,6 +407,9 @@ string any_space_to_standard_space (string s)
 {
   s = filter_string_str_replace (unicode_non_breaking_space_entity (), " ", s);
   s = filter_string_str_replace (non_breaking_space_u00A0 (), " ", s);
+  s = filter_string_str_replace (en_space_u2002 (), " ", s);
+  s = filter_string_str_replace (figure_space_u2007 (), " ", s);
+  s = filter_string_str_replace (narrow_non_breaking_space_u202F (), " ", s);
   return s;
 }
 
@@ -444,7 +478,7 @@ string narrow_non_breaking_space_u202F ()
 // Returns the length of string s in unicode points, not in bytes.
 size_t unicode_string_length (string s)
 {
-  int length = utf8::distance (s.begin(), s.end());
+  size_t length = static_cast<size_t> (utf8::distance (s.begin(), s.end()));
   return length;
 }
 
@@ -453,8 +487,8 @@ size_t unicode_string_length (string s)
 // If len = 0, the string from start till end is returned.
 string unicode_string_substr (string s, size_t pos, size_t len)
 {
-  char * input = (char *) s.c_str();
-  char * startiter = (char *) input;
+  char * input = const_cast<char *>(s.c_str());
+  char * startiter = input;
   size_t length = strlen (input);
   char * veryend = input + length + 1;
   // Iterate forward pos times.
@@ -486,8 +520,8 @@ string unicode_string_substr (string s, size_t pos, size_t len)
     len--;
   }
   // Return substring.
-  size_t startpos = startiter - input;
-  size_t lenpos = enditer - startiter;
+  size_t startpos = static_cast <size_t> (startiter - input);
+  size_t lenpos = static_cast <size_t> (enditer - startiter);
   s = s.substr (startpos, lenpos);
   return s;
 }
@@ -496,11 +530,11 @@ string unicode_string_substr (string s, size_t pos, size_t len)
 // Equivalent to PHP's mb_strpos function.
 size_t unicode_string_strpos (string haystack, string needle, size_t offset)
 {
-  int haystack_length = unicode_string_length (haystack);
-  int needle_length = unicode_string_length (needle);
-  for (int pos = offset; pos <= haystack_length - needle_length; pos++) {
-    string substring = unicode_string_substr (haystack, pos, needle_length);
-    if (substring == needle) return pos;
+  int haystack_length = static_cast<int>(unicode_string_length (haystack));
+  int needle_length = static_cast<int>(unicode_string_length (needle));
+  for (int pos = static_cast<int>(offset); pos <= haystack_length - needle_length; pos++) {
+    string substring = unicode_string_substr (haystack, static_cast <size_t> (pos), static_cast <size_t> (needle_length));
+    if (substring == needle) return static_cast <size_t> (pos);
   }
   return string::npos;
 }
@@ -512,11 +546,11 @@ size_t unicode_string_strpos_case_insensitive (string haystack, string needle, s
   haystack = unicode_string_casefold (haystack);
   needle = unicode_string_casefold (needle);
   
-  int haystack_length = unicode_string_length (haystack);
-  int needle_length = unicode_string_length (needle);
-  for (int pos = offset; pos <= haystack_length - needle_length; pos++) {
-    string substring = unicode_string_substr (haystack, pos, needle_length);
-    if (substring == needle) return pos;
+  int haystack_length = static_cast<int>(unicode_string_length (haystack));
+  int needle_length = static_cast<int>(unicode_string_length (needle));
+  for (int pos = static_cast<int>(offset); pos <= haystack_length - needle_length; pos++) {
+    string substring = unicode_string_substr (haystack, static_cast<size_t> (pos), static_cast<size_t> (needle_length));
+    if (substring == needle) return static_cast<size_t>(pos);
   }
   return string::npos;
 }
@@ -546,9 +580,9 @@ string unicode_string_casefold (string s)
       string character = unicode_string_substr (s, pos, 1);
       // Convert it to a Unicode point.
       const utf8proc_uint8_t *str = (const unsigned char *) (character.c_str ());
-      utf8proc_ssize_t len = character.length ();
+      utf8proc_ssize_t len = static_cast<utf8proc_ssize_t> (character.length ());
       utf8proc_int32_t dst;
-      utf8proc_ssize_t output = utf8proc_iterate (str, len, &dst);
+      [[maybe_unused]] utf8proc_ssize_t output = utf8proc_iterate (str, len, &dst);
       // Convert the Unicode point to lower case.
       utf8proc_int32_t luc = utf8proc_tolower (dst);
       // Convert the Unicode point back to a UTF-8 string.
@@ -593,9 +627,9 @@ string unicode_string_uppercase (string s)
       string character = unicode_string_substr (s, pos, 1);
       // Convert it to a Unicode point.
       const utf8proc_uint8_t *str = (const unsigned char *) (character.c_str ());
-      utf8proc_ssize_t len = character.length ();
+      utf8proc_ssize_t len = static_cast<utf8proc_ssize_t> (character.length ());
       utf8proc_int32_t dst;
-      utf8proc_ssize_t output = utf8proc_iterate (str, len, &dst);
+      [[maybe_unused]] utf8proc_ssize_t output = utf8proc_iterate (str, len, &dst);
       // Convert the Unicode point to lower case.
       utf8proc_int32_t luc = utf8proc_toupper (dst);
       // Convert the Unicode point back to a UTF-8 string.
@@ -630,11 +664,10 @@ string unicode_string_transliterate (string s)
     for (unsigned int pos = 0; pos < string_length; pos++) {
       string character = unicode_string_substr (s, pos, 1);
       const utf8proc_uint8_t *str = (const unsigned char *) (character.c_str ());
-      utf8proc_ssize_t len = character.length ();
+      utf8proc_ssize_t len = static_cast<utf8proc_ssize_t> (character.length ());
       uint8_t *dest;
       utf8proc_option_t options = (utf8proc_option_t) (UTF8PROC_DECOMPOSE | UTF8PROC_STRIPMARK);
-      utf8proc_ssize_t output = utf8proc_map (str, len, &dest, options);
-      (void) output;
+      [[maybe_unused]] auto output = utf8proc_map (str, len, &dest, options);
       stringstream ss;
       ss << dest;
       transliteration.append (ss.str ());
@@ -682,10 +715,9 @@ bool unicode_string_is_punctuation (string s)
     s = unicode_string_substr (s, 0, 1);
     // Convert the string to a Unicode point.
     const utf8proc_uint8_t *str = (const unsigned char *) (s.c_str ());
-    utf8proc_ssize_t len = s.length ();
+    utf8proc_ssize_t len = static_cast<utf8proc_ssize_t> (s.length ());
     utf8proc_int32_t codepoint;
-    utf8proc_ssize_t output = utf8proc_iterate (str, len, &codepoint);
-    (void) output;
+    [[maybe_unused]] auto output = utf8proc_iterate (str, len, &codepoint);
     // Get category.
     utf8proc_category_t category = utf8proc_category	(codepoint);
     if ((category >= UTF8PROC_CATEGORY_PC) && (category <= UTF8PROC_CATEGORY_PO)) return true;
@@ -713,10 +745,9 @@ int unicode_string_convert_to_codepoint (string s)
       s = unicode_string_substr (s, 0, 1);
       // Convert the string to a Unicode point.
       const utf8proc_uint8_t *str = (const unsigned char *) (s.c_str ());
-      utf8proc_ssize_t len = s.length ();
+      utf8proc_ssize_t len = static_cast<utf8proc_ssize_t> (s.length ());
       utf8proc_int32_t codepoint;
-      utf8proc_ssize_t output = utf8proc_iterate (str, len, &codepoint);
-      (void) output;
+      [[maybe_unused]] auto output = utf8proc_iterate (str, len, &codepoint);
       point = codepoint;
     } catch (...) {
     }
@@ -749,11 +780,145 @@ string unicode_string_str_replace (string search, string replace, string subject
 }
 
 
-// C++ equivalent for PHP's rand function
+#ifdef HAVE_ICU
+string icu_string_normalize (string s, bool remove_diacritics, bool casefold)
+{
+  // Skip any conversions that slow things down if no normalization is to be done.
+  if (!remove_diacritics && !casefold) return s;
+  
+  // UTF-8 std::string -> UTF-16 UnicodeString
+  UnicodeString source = UnicodeString::fromUTF8 (StringPiece (s));
+
+  // The order of doing the normalization action may be of influence on the result.
+  // Right now it seems more logical to remove diacritics first and then doing the case folding.
+
+  // Removal of diacritics.
+  if (remove_diacritics) {
+    // Transliterate UTF-16 UnicodeString following this rule:
+    // decompose, remove diacritics, recompose
+    UErrorCode status = U_ZERO_ERROR;
+    Transliterator *accents_converter = Transliterator::createInstance("NFD; [:M:] Remove; NFC", UTRANS_FORWARD, status);
+    accents_converter->transliterate(source);
+  }
+
+  // Case folding.
+  if (casefold) {
+    source.foldCase ();
+  }
+  
+  // UTF-16 UnicodeString -> UTF-8 std::string
+  string result;
+  source.toUTF8String (result);
+  
+  return result;
+}
+#endif
+
+
+// Some code for when it's necessary to find out if text is alphabetic.
+//#include <unicode/uchar.h>
+//#include <unicode/unistr.h>
+//#include <unicode/schriter.h>
+//const std::string& word = static_cast<const std::string&>(symbol);
+//icu::UnicodeString uword = icu::UnicodeString::fromUTF8(icu::StringPiece(word.data(), word.size()));
+//
+//std::string signature = "<unk";
+//
+//// signature for English, taken from Stanford parser's getSignature5
+//int num_caps = 0;
+//bool has_digit  = false;
+//bool has_dash   = false;
+//bool has_lower  = false;
+//bool has_punct  = false;
+//bool has_symbol = false;
+//
+//size_t length = 0;
+//UChar32 ch0 = 0;
+//UChar32 ch_1 = 0;
+//UChar32 ch_2 = 0;
+//
+//icu::StringCharacterIterator iter(uword);
+//for (iter.setToStart(); iter.hasNext(); ++ length) {
+//  const UChar32 ch = iter.next32PostInc();
+//
+//  // keep initial char...
+//  if (ch0 == 0)
+//    ch0 = ch;
+//
+//  ch_2 = ch_1;
+//  ch_1 = ch;
+//
+//  const int32_t gc = u_getIntPropertyValue(ch, UCHAR_GENERAL_CATEGORY_MASK);
+//
+//  has_dash   |= ((gc & U_GC_PD_MASK) != 0);
+//  has_punct  |= ((gc & U_GC_P_MASK) != 0);
+//  has_symbol |= ((gc & U_GC_S_MASK) != 0);
+//
+//  has_digit  |= (u_getNumericValue(ch) != U_NO_NUMERIC_VALUE);
+//
+//  if (u_isUAlphabetic(ch)) {
+//    if (u_isULowercase(ch))
+//      has_lower = true;
+//    else if (u_istitle(ch)) {
+//      has_lower = true;
+//      ++ num_caps;
+//    } else
+//      ++ num_caps;
+//  }
+//}
+//
+//// transform into lower...
+//uword.toLower();
+//ch_2 = (ch_2 ? u_tolower(ch_2) : ch_2);
+//ch_1 = (ch_1 ? u_tolower(ch_1) : ch_1);
+//
+//// we do not check loc...
+//if (u_isUUppercase(ch0) || u_istitle(ch0))
+//  signature += "-caps";
+//else if (! u_isUAlphabetic(ch0) && num_caps)
+//  signature += "-caps";
+//else if (has_lower)
+//  signature += "-lc";
+//
+//if (has_digit)
+//  signature += "-num";
+//if (has_dash)
+//  signature += "-dash";
+//if (has_punct)
+//  signature += "-punct";
+//if (has_symbol)
+//  signature += "-sym";
+//
+//if (length >= 3 && ch_1 == 's') {
+//  if (ch_2 != 's' && ch_2 != 'i' && ch_2 != 'u')
+//    signature += "-s";
+//} else if (length >= 5 && ! has_dash && ! (has_digit && num_caps > 0)) {
+//  if (uword.endsWith("ed"))
+//    signature += "-ed";
+//  else if (uword.endsWith("ing"))
+//    signature += "-ing";
+//  else if (uword.endsWith("ion"))
+//    signature += "-ion";
+//  else if (uword.endsWith("er"))
+//    signature += "-er";
+//  else if (uword.endsWith("est"))
+//    signature += "-est";
+//  else if (uword.endsWith("ly"))
+//    signature += "-ly";
+//  else if (uword.endsWith("ity"))
+//    signature += "-ity";
+//  else if (uword.endsWith("y"))
+//    signature += "-y";
+//  else if (uword.endsWith("al"))
+//    signature += "-al";
+//}
+
+
+// Generate a truly random number between $floor and $ceiling.
 int filter_string_rand (int floor, int ceiling)
 {
   int range = ceiling - floor;
-  int r = rand () % range + floor;
+  int r = config_globals_int_distribution (config_globals_random_engine) % range + floor;
   return r;
 }
 
@@ -873,7 +1038,13 @@ string get_checkbox_status (bool enabled)
 string get_disabled (bool disabled)
 {
   if (disabled) return "disabled";
-  return "";
+  return string();
+}
+
+
+string get_reload ()
+{
+  return string("reload");
 }
 
 
@@ -1237,7 +1408,7 @@ vector <string> filter_string_search_needles (string search, string text)
 // Returns an integer identifier based on the name of the current user.
 int filter_string_user_identifier (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   string username = request->session_logic()->currentUser ();
   string hash = md5 (username).substr (0, 5);
   int identifier = my_stoi (hash, NULL, 36);
@@ -1270,7 +1441,7 @@ string hex2bin (string hex)
     for (string::const_iterator pos = hex.begin(); pos < hex.end(); pos += 2)
     {
       extract.assign (pos, pos+2);
-      out.push_back (my_stoi (extract, nullptr, 16));
+      out.push_back (static_cast<char> (my_stoi (extract, nullptr, 16)));
     }
   }
   return out;
@@ -1303,7 +1474,7 @@ string convert_xml_character_entities_to_characters (string data)
 {
   bool keep_going = true;
   int iterations = 0;
-  size_t pos1 = -1;
+  size_t pos1 = static_cast<size_t>(-1);
   do {
     iterations++;
     pos1 = data.find ("&#x", pos1 + 1);
@@ -1330,11 +1501,25 @@ string convert_xml_character_entities_to_characters (string data)
     int cp = codepoint;
     // Adapted from: http://www.zedwood.com/article/cpp-utf8-char-to-codepoint.
     char c[5]={ 0x00,0x00,0x00,0x00,0x00 };
-    if     (cp<=0x7F) { c[0] = cp;  }
-    else if(cp<=0x7FF) { c[0] = (cp>>6)+192; c[1] = (cp&63)+128; }
-    else if(0xd800<=cp && cp<=0xdfff) {} //invalid block of utf8
-    else if(cp<=0xFFFF) { c[0] = (cp>>12)+224; c[1]= ((cp>>6)&63)+128; c[2]=(cp&63)+128; }
-    else if(cp<=0x10FFFF) { c[0] = (cp>>18)+240; c[1] = ((cp>>12)&63)+128; c[2] = ((cp>>6)&63)+128; c[3]=(cp&63)+128; }
+    if (cp<=0x7F) {
+      c[0] = static_cast<char> (cp);
+    }
+    else if (cp<=0x7FF) {
+      c[0] = static_cast<char>((cp>>6)+192);
+      c[1] = static_cast<char>((cp&63)+128);
+    }
+    else if (0xd800<=cp && cp<=0xdfff) {} // Invalid block of utf8.
+    else if (cp<=0xFFFF) {
+      c[0] = static_cast<char>((cp>>12)+224);
+      c[1] = static_cast<char>(((cp>>6)&63)+128);
+      c[2] = static_cast<char>((cp&63)+128);
+    }
+    else if (cp<=0x10FFFF) {
+      c[0] = static_cast<char>((cp>>18)+240);
+      c[1] = static_cast<char>(((cp>>12)&63)+128);
+      c[2] = static_cast<char>(((cp>>6)&63)+128);
+      c[3] = static_cast<char>((cp&63)+128);
+    }
     string u8str = string (c);
     
     data.insert (pos1, u8str);
@@ -1361,8 +1546,8 @@ string encrypt_decrypt (string key, string data)
 // Gets a new random string for sessions, encryption, you name it.
 string get_new_random_string ()
 {
-  string u = convert_to_string (filter_date_numerical_microseconds ());
-  string s = convert_to_string (filter_date_seconds_since_epoch ());
+  string u = convert_to_string (filter::date::numerical_microseconds ());
+  string s = convert_to_string (filter::date::seconds_since_epoch ());
   string r = convert_to_string (config_globals_int_distribution (config_globals_random_engine));
   return md5 (u + s + r);
 }
@@ -1467,7 +1652,7 @@ void array_move_from_to (vector <string> & container, size_t from, size_t to)
   // Put the data into a map where the keys are multiplied by two.
   map <int, string> mapped_container;
   for (unsigned int i = 0; i < container.size(); i++) {
-    mapped_container [i * 2] = container [i];
+    mapped_container [static_cast<int>(i * 2)] = container [i];
   }
 
   // Direction of moving.
@@ -1478,11 +1663,11 @@ void array_move_from_to (vector <string> & container, size_t from, size_t to)
   to *= 2;
   
   // Remove the item, and insert it by a key that puts it at the desired position.
-  string moving_item = mapped_container [from];
-  mapped_container.erase (from);
+  string moving_item = mapped_container [(int)from];
+  mapped_container.erase ((int)from);
   if (move_up) to++;
   else to--;
-  mapped_container [to] = moving_item;
+  mapped_container [(int)to] = moving_item;
   
   // Since the map sorts by key,
   // transfer its data back to the original container.
@@ -1548,3 +1733,61 @@ string filter_text_html_get_element (string html, string element)
   }
   return html;
 }
+
+
+string filter_string_tidy_invalid_html (string html)
+{
+  // Everything in the <head> can be left out: It is not relevant.
+  filter_string_replace_between (html, "<head>", "</head>", "");
+  
+  // Every <script...</script> can be left out: They are irrelevant.
+  int counter = 0;
+  while (counter < 100) {
+    counter++;
+    bool replaced = filter_string_replace_between (html, "<script", "</script>", "");
+    if (!replaced) break;
+  }
+  
+#ifdef HAVE_CLOUD
+
+  // Create a parser context.
+  htmlParserCtxtPtr parser = htmlCreatePushParserCtxt (NULL, NULL, NULL, 0, NULL, XML_CHAR_ENCODING_UTF8);
+  
+  // Set relevant options on the parser context.
+  htmlCtxtUseOptions(parser, HTML_PARSE_NOBLANKS | HTML_PARSE_NOERROR | HTML_PARSE_NOWARNING | HTML_PARSE_NONET);
+
+  // Parse the (X)HTML text.
+  // char * data : buffer containing part of the web page
+  // int len : number of bytes in data
+  // Last argument is 0 if the web page isn't complete, and 1 for the final call.
+  htmlParseChunk(parser, html.c_str(), (int)html.size(), 1);
+
+  // Extract the fixed html
+  if (parser->myDoc) {
+    xmlChar *s;
+    int size;
+    xmlDocDumpMemory(parser->myDoc, &s, &size);
+    html = (char *)s;
+    xmlFree(s);
+  }
+  
+  // Free memory.
+  if (parser) xmlFree (parser);
+
+#endif
+
+  return html;
+}
+
+
+string filter_string_collapse_whitespace (string s)
+{
+  int count;
+  int iterator = 0;
+  do {
+    count = 0;
+    s = filter_string_str_replace ("  ", " ", s, &count);
+    iterator++;
+  } while ((count > 0) && iterator < 5);
+  return s;
+}
