@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,8 +17,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 
-#ifndef INCLUDED_FILTER_TEXT_H
-#define INCLUDED_FILTER_TEXT_H
+#pragma once
 
 
 #include <config/libraries.h>
@@ -28,29 +27,24 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <html/text.h>
 #include <text/text.h>
 #include <esword/text.h>
+#include <tbsx/text.h>
+#include <filter/note.h>
 
 
-class Filter_Text_Passage_Marker_Value
+namespace filter::text {
+
+class passage_marker_value
 {
 public:
-  Filter_Text_Passage_Marker_Value (int book_in, int chapter_in, string verse_in, string marker_in, string value_in);
-  int book;
-  int chapter;
-  string verse;
-  string marker;
-  string value;
+  passage_marker_value (int book, int chapter, string verse, string marker, string value);
+  int m_book;
+  int m_chapter;
+  string m_verse;
+  string m_marker;
+  string m_value;
 };
 
-
-class Filter_Text_Note_Citation
-{
-public:
-  Filter_Text_Note_Citation ();
-  Filter_Text_Note_Citation (vector <string> sequence_in, string restart_in);
-  vector <string> sequence;
-  string restart;
-  unsigned int pointer;
-};
+}
 
 
 class Filter_Text
@@ -63,9 +57,10 @@ private:
   string bible;
 
 public:
-  void addUsfmCode (string usfm);
+  void add_usfm_code (string usfm);
 private:
-  vector <string> usfmMarkersAndText; // Vector holding USFM, alternating between markup and text.
+  // Container holding USFM, alternating between markup and text.
+  vector <string> usfmMarkersAndText;
   unsigned int usfmMarkersAndTextPointer;
   bool unprocessedUsfmCodeAvailable ();
   void getUsfmNextChapter ();
@@ -73,56 +68,71 @@ private:
 public:
   void run (string stylesheet);
 private:
-  vector <string> chapterUsfmMarkersAndText; // Vector holding a chapter of USFM code, alternating between USFM and text.
-  unsigned int chapterUsfmMarkersAndTextPointer;
+  // Container holding a chapter of USFM code, alternating between USFM and text.
+  vector <string> chapter_usfm_markers_and_text;
+  unsigned int chapter_usfm_markers_and_text_pointer;
 
 public:
   void getStyles (string stylesheet);
 private:
-  map <string, Database_Styles_Item> styles; // A map of marker -> object with style information.
-  string chapterMarker; // Usually this is: c
-  vector <string> createdStyles; // Array holding styles created in Odf_Text class.
+  // A map of marker -> object with style information.
+  map <string, Database_Styles_Item> styles;
+  // Usually this is: c
+  string chapterMarker;
+  // Array holding styles created in Odf_Text class.
+  vector <string> createdStyles;
 
 public:
-  void preprocessingStage ();
+  void pre_process_usfm ();
 private:
-  int currentBookIdentifier; // Book identifier, e.g. 1, 2, 3, and so on.
-  int currentChapterNumber; // Chapter number, e.g. 1, 2, 3, etc.
-  string currentVerseNumber; // Verse number, e.g. "0", "1", "2", and so on.
+  // Book identifier, e.g. 1, 2, 3, and so on.
+  int currentBookIdentifier;
+  // Chapter number, e.g. 1, 2, 3, etc.
+  int currentChapterNumber;
+  // Verse number, e.g. "0", "1", "2", and so on.
+  string currentVerseNumber;
   string getCurrentPassageText ();
-  map <int, int> numberOfChaptersPerBook; // Map of (book, chapter number).
-  void processUsfm ();
+  // Map of (book, chapter number).
+  map <int, int> numberOfChaptersPerBook;
+  void process_usfm ();
   void processNote ();
-  void newParagraph (Database_Styles_Item style, bool keepWithNext);
+  // Opening a new paragraph.
+  void new_paragraph (const Database_Styles_Item & style, bool keepWithNext);
   void applyDropCapsToCurrentParagraph (int dropCapsLength);
   void putChapterNumberInFrame (string chapterText);
-  void createNoteCitation (Database_Styles_Item style);
-  string getNoteCitation (Database_Styles_Item style);
-  void resetNoteCitations (string moment);
-  void ensureNoteParagraphStyle (string marker, Database_Styles_Item style);
+  string getNoteCitation (const Database_Styles_Item & style);
+  void ensureNoteParagraphStyle (string marker, const Database_Styles_Item & style);
 
 public:
-  vector <Filter_Text_Passage_Marker_Value> runningHeaders; // Vector with objects (book, chapter, verse, marker, header value).
-  vector <Filter_Text_Passage_Marker_Value> longTOCs; // Vector with objects (book, chapter, verse, marker, TOC value).
-  vector <Filter_Text_Passage_Marker_Value> shortTOCs; // Vector with objects (book, chapter, verse, marker, TOC value).
-  vector <Filter_Text_Passage_Marker_Value> bookAbbreviations; // Vector with objects (book, chapter, verse, marker, abbreviation value).
+  // Container with objects (book, chapter, verse, marker, header value).
+  vector <filter::text::passage_marker_value> runningHeaders;
+  // Container with objects (book, chapter, verse, marker, TOC value).
+  vector <filter::text::passage_marker_value> longTOCs;
+  // Container with objects (book, chapter, verse, marker, TOC value).
+  vector <filter::text::passage_marker_value> shortTOCs;
+  // Container with objects (book, chapter, verse, marker, abbreviation value).
+  vector <filter::text::passage_marker_value> bookAbbreviations;
 
 public:
   // Vector with objects (book, chapter, verse, marker, label value).
-  vector <Filter_Text_Passage_Marker_Value> chapterLabels;
+  vector <filter::text::passage_marker_value> chapterLabels;
   // Vector with object (book, chapter, verse, marker, marker value).
-  vector <Filter_Text_Passage_Marker_Value> publishedChapterMarkers;
+  vector <filter::text::passage_marker_value> publishedChapterMarkers;
   // Vector with object (book, chapter, verse, marker, marker value).
-  vector <Filter_Text_Passage_Marker_Value> publishedVerseMarkers;
+  vector <filter::text::passage_marker_value> publishedVerseMarkers;
 private:
   // String holding the chapter number or text to output at the first verse.
-  string outputChapterTextAtFirstVerse;
+  string output_chapter_text_at_first_verse;
 
 public:
-  Odf_Text * odf_text_standard; // Object for creating OpenDocument with text in standard form.
-  Odf_Text * odf_text_text_only; // Object for creating OpenDocument with text only.
-  Odf_Text * odf_text_text_and_note_citations; // Object for creating OpenDocument with text and note citations.
-  Odf_Text * odf_text_notes; // Object for creating OpenDocument with the notes only.
+  // Object for creating OpenDocument with text in standard form.
+  odf_text * odf_text_standard;
+  // Object for creating OpenDocument with text only.
+  odf_text * odf_text_text_only;
+  // Object for creating OpenDocument with text and note citations.
+  odf_text * odf_text_text_and_note_citations;
+  // Object for creating OpenDocument with the notes only.
+  odf_text * odf_text_notes;
 
 public:
   void produceInfoDocument (string path);
@@ -132,30 +142,41 @@ public:
 private:
   void addToInfo (string text, bool next = false);
   void addToFallout (string text, bool next = false);
-  void addToWordList (vector <string>  & list);
+  void addToWordList (vector <string> & list);
   vector <string> wordListGlossaryDictionary;
   vector <string> hebrewWordList;
   vector <string> greekWordList;
   vector <string> subjectIndex;
 
 private:
-  map <string, Filter_Text_Note_Citation> notecitations; // Information for the citations for the notes.
+  // Information for the citations for the notes.
+  filter::note::citations note_citations;
+
   string standardContentMarkerFootEndNote;
   string standardContentMarkerCrossReference;
 
 public:
-  Html_Text * html_text_standard; // Object for creating standard web documents.
-  Html_Text * html_text_linked; // Object for creating interlinked web documents.
+  // Object for creating standard web documents.
+  Html_Text * html_text_standard;
+  // Object for creating interlinked web documents.
+  Html_Text * html_text_linked;
 
 public:
-  OnlineBible_Text * onlinebible_text; // Object for creating the input file for the Online Bible compiler.
+  // Object for creating the input file for the Online Bible compiler.
+  OnlineBible_Text * onlinebible_text;
 
 public:
-  Esword_Text * esword_text; // Object for creating the Bible module for eSword.
+  // Object for creating the Bible module for eSword.
+  Esword_Text * esword_text;
 
 public:
-  Text_Text * text_text; // Object for exporting to plain text.
+  // Object for exporting to plain text.
+  Text_Text * text_text;
 
+public:
+  // Object for exporting to TBS online bible format.
+  Tbsx_Text * tbsx_text;
+  
 public:
   void initializeHeadingsAndTextPerVerse (bool start_text_now);
   map <int, string> getVersesText ();
@@ -193,7 +214,18 @@ private:
   string notes_plain_text_buffer;
   // Handler.
   void notes_plain_text_handler ();
+  
+public:
+  // The images that have been encountered to use.
+  vector <string> image_sources;
+private:
+  // Flag for whether the processor is now within figure markup.
+  bool is_within_figure_markup = false;
+  
+public:
+private:
+  // Flag for whether to left-align certain poetry styles
+  // in exports to OpenDocument format.
+  bool odt_left_align_verse_in_poetry_styles = false;
+  
 };
-
-
-#endif
