@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2019 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -17,13 +17,10 @@
  */
 
 
-#ifndef INCLUDED_FONTS_LOGIC_H
-#define INCLUDED_FONTS_LOGIC_H
-
+#pragma once
 
 #include <config/libraries.h>
 
-
 class Fonts_Logic
 {
 public:
@@ -36,6 +33,3 @@ public:
 private:
   static string folder ();
 };
-
-
-#endif
