@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,10 +23,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <config/globals.h>
 #include <filter/url.h>
 #include <filter/string.h>
+#include <filter/date.h>
 #include <thread>
 #include <timer/index.h>
 #include <database/config/general.h>
+#include <database/config/bible.h>
 #include <database/logs.h>
+#include <database/mappings.h>
+#include <database/books.h>
 #include <setup/index.h>
 #include <setup/logic.h>
 #include <library/locks.h>
@@ -37,6 +41,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <sendreceive/logic.h>
 #include <ldap/logic.h>
 #include <locale/logic.h>
+#include <ipc/focus.h>
+#include <client/logic.h>
 
 
 bool bibledit_started = false;
@@ -50,11 +56,29 @@ const char * bibledit_get_version_number ()
 
 
 // Get the port number that Bibledit's web server listens on.
-string library_bibledit_network_port;
+// If the server already runs, it will give that port number.
+// If the server does not yet run, on a client, it will negotiate a free port number, and give that.
 const char * bibledit_get_network_port ()
 {
-  library_bibledit_network_port = config_logic_http_network_port ();
-  return library_bibledit_network_port.c_str ();
+  // If the port number has already been set or negotiated, return that port number.
+  if (!config_globals_negotiated_port_number.empty()) return config_globals_negotiated_port_number.c_str();
+
+  // On a client device, negotiate a local port number.
+#ifdef HAVE_CLIENT
+  vector <int> ports = { 9876, 9987, 9998 };
+  for (auto port : ports) {
+    if (!filter_url_port_can_connect ("localhost", port)) {
+      config_globals_negotiated_port_number = convert_to_string(port);
+      break;
+    }
+  }
+#endif
+
+  // Set the port number.
+  config_logic_http_network_port ();
+
+  // Give the port number to the caller.
+  return config_globals_negotiated_port_number.c_str ();
 }
 
 
@@ -103,8 +127,7 @@ void bibledit_initialize_library (const char * package, const char * webroot)
   int hours = 0;
 #ifdef HAVE_WINDOWS
   TIME_ZONE_INFORMATION tzi;
-  DWORD dwRet = GetTimeZoneInformation (&tzi);
-  (void)dwRet;
+  [[maybe_unused]] auto dwRet = GetTimeZoneInformation (&tzi);
   hours = 0 - (tzi.Bias / 60);
 #else
   // Set local timezone offset in the library on Linux.
@@ -206,7 +229,7 @@ void bibledit_start_library ()
 // Gets the last page that was opened via the menu.
 const char * bibledit_get_last_page ()
 {
-  string href = Database_Config_General::getLastMenuClick ();
+  static string href = Database_Config_General::getLastMenuClick ();
   return href.c_str();
 }
 
@@ -301,12 +324,15 @@ void bibledit_stop_library ()
   filter_url_http_get (url, error, false);
 
 #ifdef RUN_SECURE_SERVER
-  // Connect to the secure server to initiate its shutdown mechanism.
-  url = "https://localhost:";
-  url.append (convert_to_string (config_logic_https_network_port ()));
-  filter_url_http_get (url, error, false);
-  // Let the connection start, then close it.
-  // The server will then abort the TLS handshake, and shut down.
+  // If the secure server runs, connect to it to initiate its shutdown mechanism.
+  string https_port = config_logic_https_network_port ();
+  if (https_port.length() > 1) {
+    url = "https://localhost:";
+    url.append (https_port);
+    filter_url_http_get (url, error, false);
+    // Let the connection start, then close it.
+    // The server will then abort the TLS handshake, and shut down.
+  }
 #endif
 
 #ifndef HAVE_ANDROID
@@ -371,3 +397,102 @@ const char * bibledit_disable_selection_popup_chrome_os ()
   }
   return "false";
 }
+
+
+// https://github.com/bibledit/cloud/issues/437
+// Accordance expects to receive a standardized verse reference.
+// So, for instance, a reference of Psalm 13:3 in the Hebrew Bible
+// will instead become the standardized (KJV-like) Psalm 13:2.
+const char * bibledit_get_reference_for_accordance ()
+{
+  // Keep the static reference always in memory as a global reference.
+  // The purpose is that the value remains live in memory for the caller,
+  // even after the function has returned, and local variables will have been destroyed.
+  static string reference;
+  
+  // Wait till all the data has been initialized.
+  // If the data is not yet initialized, return an empty reference instead.
+  if (!config_globals_data_initialized) return reference.c_str();
+
+  // Get the username on this client device.
+  string user = client_logic_get_username ();
+
+  // Get the active Bible and its versification system.
+  Webserver_Request request;
+  request.session_logic()->set_username(user);
+  Database_Config_User database_config_user (&request);
+  string bible = request.database_config_user ()->getBible ();
+  string versification = Database_Config_Bible::getVersificationSystem (bible);
+
+  int book = Ipc_Focus::getBook (&request);
+  int chapter = Ipc_Focus::getChapter (&request);
+  int verse = Ipc_Focus::getVerse (&request);
+
+  // Accordance expects a verse reference in the English versification system.
+  vector <Passage> passages;
+  Database_Mappings database_mappings;
+  if ((versification != english()) && !versification.empty ()) {
+    passages = database_mappings.translate (versification, english (), book, chapter, verse);
+  } else {
+    passages.push_back (Passage ("", book, chapter, convert_to_string (verse)));
+  }
+  if (passages.empty()) return "";
+
+  // Accordance expects for instance, 2 Corinthians 9:2, to be broadcast as "2CO 9:2".
+  book = passages[0].book;
+  chapter = passages[0].chapter;
+  string verse_s = passages[0].verse;
+  string usfm_id = Database_Books::getUsfmFromId (book);
+  reference = usfm_id + " " + convert_to_string (chapter) + ":" + convert_to_string (verse_s);
+
+  // Return the reference.
+  return reference.c_str ();
+}
+
+
+// https://github.com/bibledit/cloud/issues/437
+// Accordance sends a standardized verse reference.
+// So, for instance, a reference of Psalm 13:3 in the Hebrew Bible
+// will instead become the standardized (KJV-like) Psalm 13:2.
+void bibledit_put_reference_from_accordance (const char * reference)
+{
+  // Get and set the user name on this client device.
+  string user = client_logic_get_username ();
+  Webserver_Request request;
+  request.session_logic()->set_username(user);
+
+  // Setting whether to enable receiving verse references from Accordance.
+  bool enabled  = request.database_config_user ()->getReceiveFocusedReferenceFromAccordance ();
+  if (!enabled) return;
+  
+  // Interpret the passage from Accordance, e.g. MAT 1:1.
+  // Accordance broadcasts for instance, 2 Corinthians 9:2, as "2CO 9:2".
+  vector<string> book_rest = filter_string_explode (reference, ' ');
+  if (book_rest.size() != 2) return;
+  int book = Database_Books::getIdFromUsfm (book_rest[0]);
+  vector <string> chapter_verse = filter_string_explode(book_rest[1], ':');
+  if (chapter_verse.size() != 2) return;
+  int chapter = convert_to_int(chapter_verse[0]);
+  int verse = convert_to_int(chapter_verse[1]);
+
+  // Get the active Bible and its versification system.
+  Database_Config_User database_config_user (&request);
+  string bible = request.database_config_user ()->getBible ();
+  string versification = Database_Config_Bible::getVersificationSystem (bible);
+
+  // Accordance expects a verse reference in the English versification system.
+  vector <Passage> passages;
+  Database_Mappings database_mappings;
+  if ((versification != english()) && !versification.empty ()) {
+    passages = database_mappings.translate (english (), versification, book, chapter, verse);
+  } else {
+    passages.push_back (Passage ("", book, chapter, convert_to_string (verse)));
+  }
+  if (passages.empty()) return;
+
+  // Set the focused passage in Bibledit.
+  book = passages[0].book;
+  chapter = passages[0].chapter;
+  string verse_s = passages[0].verse;
+  Ipc_Focus::set (&request, book, chapter, verse);
+}
