@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2019 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include <tasks/logic.h>
 #include <locale/translate.h>
 #include <bb/logic.h>
+#include <client/logic.h>
 
 
 string Paratext_Logic::searchProjectsFolder ()
@@ -51,7 +52,7 @@ string Paratext_Logic::searchProjectsFolder ()
     vector <string> files = filter_url_scandir (homedir);
     for (auto file : files) {
       if (file.find ("Paratext") != string::npos) {
-        return filter_url_create_path (homedir, file);
+        return filter_url_create_path ({homedir, file});
       }
     }
   }
@@ -62,7 +63,7 @@ string Paratext_Logic::searchProjectsFolder ()
   vector <string> files = filter_url_scandir (homedir);
   for (auto file : files) {
     if (file.find ("Paratext") != string::npos) {
-      string path = filter_url_create_path (homedir, file);
+      string path = filter_url_create_path ({homedir, file});
       path = filter_string_str_replace ("\\\\", "\\", path);
       return path;
     }
@@ -79,7 +80,7 @@ vector <string> Paratext_Logic::searchProjects (string projects_folder)
   vector <string> projects;
   vector <string> folders = filter_url_scandir (projects_folder);
   for (auto folder : folders) {
-    string path = filter_url_create_path (projects_folder, folder);
+    string path = filter_url_create_path ({projects_folder, folder});
     if (filter_url_is_dir (path)) {
       map <int, string> books = searchBooks (path);
       if (!books.empty ()) projects.push_back (folder);
@@ -96,7 +97,7 @@ map <int, string> Paratext_Logic::searchBooks (string project_path)
   for (auto file : files) {
     if (file.find (".BAK") != string::npos) continue;
     if (file.find ("~") != string::npos) continue;
-    string path = filter_url_create_path (project_path, file);
+    string path = filter_url_create_path ({project_path, file});
     int id = getBook (path);
     if (id) books [id] = file;
   }
@@ -177,7 +178,7 @@ void Paratext_Logic::copyBibledit2Paratext (string bible)
     
     if (!paratext_book.empty ()) {
 
-      string path = filter_url_create_path (paratext_project_folder, paratext_book);
+      string path = filter_url_create_path ({paratext_project_folder, paratext_book});
       Database_Logs::log (bookname + ": " "Saving to:" " " + path);
       // Paratext on Windows and on Linux store the line ending with carriage return and line feed.
       filter_url_file_put_contents (path, lf2crlf (usfm));
@@ -222,7 +223,7 @@ void Paratext_Logic::copyParatext2Bibledit (string bible)
     string bookname = Database_Books::getEnglishFromId (book);
 
     string paratext_book = element.second;
-    string path = filter_url_create_path (projectFolder (bible), paratext_book);
+    string path = filter_url_create_path ({projectFolder (bible), paratext_book});
 
     Database_Logs::log (bookname + ": " "Scheduling import from:" " " + path);
 
@@ -240,7 +241,7 @@ void Paratext_Logic::copyParatext2Bibledit (string bible)
 
 string Paratext_Logic::projectFolder (string bible)
 {
-  return filter_url_create_path (Database_Config_General::getParatextProjectsFolder (), Database_Config_Bible::getParatextProject (bible));
+  return filter_url_create_path ({Database_Config_General::getParatextProjectsFolder (), Database_Config_Bible::getParatextProject (bible)});
 }
 
 
@@ -260,9 +261,9 @@ string Paratext_Logic::ancestor (string bible, int book)
 
 string Paratext_Logic::ancestorPath (string bible, int book)
 {
-  string path = filter_url_create_root_path ("paratext", "ancestors", bible);
+  string path = filter_url_create_root_path ({"paratext", "ancestors", bible});
   if (!file_or_dir_exists (path)) filter_url_mkdir (path);
-  if (book) path = filter_url_create_path (path, convert_to_string (book));
+  if (book) path = filter_url_create_path ({path, convert_to_string (book)});
   return path;
 }
 
@@ -281,7 +282,7 @@ vector <string> Paratext_Logic::enabledBibles ()
 }
 
 
-void Paratext_Logic::synchronize ()
+void Paratext_Logic::synchronize (tasks::enums::paratext_sync method)
 {
   // The Bibles for which Paratext synchronization has been enabled.
   vector <string> bibles = enabledBibles ();
@@ -294,14 +295,9 @@ void Paratext_Logic::synchronize ()
   Database_Logs::log (synchronizeStartText (), Filter_Roles::translator ());
   
   
-  string username;
-  {
-    Database_Users database_users;
-    vector <string> users = database_users.getUsers ();
-    if (users.empty()) username = "unknown";
-    else username = users [0];
-  }
+  string username = client_logic_get_username ();
 
+  
   // When Bibledit writes changes to Paratext's USFM files, 
   // Paratext does not reload those changed USFM files. 
   // Thus Bibledit may overwrite changes made by others in the loaded chapter in Paratext.
@@ -341,6 +337,7 @@ void Paratext_Logic::synchronize ()
       string paratext_book = paratext_books [book];
       if (paratext_book.empty ()) {
         Database_Logs::log (journalTag (bible, book, -1) + "The Paratext project does not have this book", Filter_Roles::translator ());
+        Database_Logs::log (journalTag (bible, book, -1) + "Looked for a file with " + usfm_get_opening_usfm("id") + Database_Books::getUsfmFromId (book) + " on the first line", Filter_Roles::translator ());
         continue;
       }
       
@@ -361,7 +358,7 @@ void Paratext_Logic::synchronize ()
       // Remove the carriage return that Paratext stores on both Windows and Linux.
       map <int, string> paratext_usfm;
       {
-        string path = filter_url_create_path (projectFolder (bible), paratext_book);
+        string path = filter_url_create_path ({projectFolder (bible), paratext_book});
         string book_usfm = crlf2lf (filter_url_file_get_contents (path));
         // Paratext on Linux has been seen adding empty lines right after \c (chapter).
         // It does that after syncing with Bibledit and editing the chapter in Paratext.
@@ -391,22 +388,51 @@ void Paratext_Logic::synchronize ()
       
       for (int chapter : chapters) {
         
-        string usfm;
         string ancestor = ancestor_usfm [chapter];
         string bibledit = database_bibles.getChapter (bible, book, chapter);
         string paratext = paratext_usfm [chapter];
 
+        // Results of the merge or copy operations.
         vector <string> messages;
-        bool merged;
         vector <Merge_Conflict> conflicts;
-
-        // Run the synchronizer.
-        usfm = synchronize (ancestor, bibledit, paratext, messages, merged, conflicts);
+        string updated_usfm;
         
-        // If there was a result of syncing, set the ancestor and paratext data.
-        if (!usfm.empty ()) {
-          ancestor_usfm [chapter] = usfm;
-          paratext_usfm [chapter] = usfm;
+        // Handle the synchronization method.
+        switch (method) {
+          case tasks::enums::paratext_sync::none:
+          {
+            break;
+          }
+          case tasks::enums::paratext_sync::bi_directional:
+          {
+            // Run the merge synchronizer.
+            updated_usfm = synchronize (ancestor, bibledit, paratext, messages, conflicts);
+            break;
+          }
+          case tasks::enums::paratext_sync::bibledit_to_paratext:
+          {
+            // If there's a change between Bibledit and Paratext, take the Bibledit data.
+            if (bibledit != paratext) {
+              updated_usfm = bibledit;
+              messages.push_back (translate ("Copy Bibledit to Paratext"));
+            }
+            break;
+          }
+          case tasks::enums::paratext_sync::paratext_to_bibledit:
+          {
+            // If there's a change between Paratext and Bibledit, take the Paratext data.
+            if (paratext != bibledit) {
+              updated_usfm = paratext;
+              messages.push_back (translate ("Copy Paratext to Bibledit"));
+            }
+            break;
+          }
+        }
+
+        // If there was a result of syncing or copying, set the ancestor and paratext data.
+        if (!updated_usfm.empty ()) {
+          ancestor_usfm [chapter] = updated_usfm;
+          paratext_usfm [chapter] = updated_usfm;
         }
         
         // Messages for the logbook.
@@ -414,23 +440,24 @@ void Paratext_Logic::synchronize ()
           Database_Logs::log (journalTag (bible, book, chapter) + message, Filter_Roles::translator ());
         }
 
-        // Log the change due to a merge.
-        if (merged) {
-          bible_logic_log_change (bible, book, chapter, usfm, "", "Paratext", true);
+        // Log the change due to a merge or copy.
+        if (!updated_usfm.empty ()) {
+          bible_logic_log_change (bible, book, chapter, updated_usfm, "", "Paratext", true);
         }
 
         // If there's any conflicts, email full details about the conflict to the user.
         // This may enable the user to resolve conflicts manually.
+        filter_merge_add_book_chapter (conflicts, book, chapter);
         bible_logic_merge_irregularity_mail ({ username }, conflicts);
         
         // Store the updated chapter in Bibledit.
-        if (!usfm.empty ()) {
+        if (!updated_usfm.empty ()) {
           // Set flag for saving to Paratext.
           book_is_updated = true;
           // Store it only in case the Bibledit data was updated.
           // https://github.com/bibledit/cloud/issues/339
-          if (usfm != bibledit) {
-            bible_logic_store_chapter (bible, book, chapter, usfm);
+          if (updated_usfm != bibledit) {
+            bible_logic_store_chapter (bible, book, chapter, updated_usfm);
           }
         }
 
@@ -460,7 +487,7 @@ void Paratext_Logic::synchronize ()
             usfm.append (filter_string_trim (data));
           }
         }
-        string path = filter_url_create_path (projectFolder (bible), paratext_book);
+        string path = filter_url_create_path ({projectFolder (bible), paratext_book});
         filter_url_file_put_contents (path, lf2crlf (usfm));
       }
     }
@@ -472,12 +499,11 @@ void Paratext_Logic::synchronize ()
 
 string Paratext_Logic::synchronize (string ancestor, string bibledit, string paratext,
                                     vector <string> & messages,
-                                    bool & merged, vector <Merge_Conflict> & conflicts)
+                                    vector <Merge_Conflict> & conflicts)
 {
   string resulting_usfm;
 
   messages.clear ();
-  merged = false;
   conflicts.clear ();
   
   // If Bibledit has the chapter, and Paratext does not, take the Bibledit chapter.
@@ -515,7 +541,6 @@ string Paratext_Logic::synchronize (string ancestor, string bibledit, string par
   else if (!ancestor.empty ()) {
     resulting_usfm = filter_merge_run (ancestor, bibledit, paratext, true, conflicts);
     messages.push_back (translate ("Chapter merged"));
-    merged = true;
   }
   
   // Cannot merge the two.
