@@ -36,6 +36,7 @@
 #include <fonts/logic.h>
 #include <navigation/passage.h>
 #include <dialog/list.h>
+#include <dialog/list2.h>
 #include <ipc/focus.h>
 #include <menu/logic.h>
 #include <bb/logic.h>
@@ -81,7 +82,13 @@ string read_index (void * webserver_request)
     // Specific configuration for the Indonesian free Cloud instance.
     // The name of the default Bible in the Read tab will be AlkitabKita
     // (That means Our/Everyone's Translation.
-    request->database_config_user()->setBible (filter_indonesian_alkitabkita_ourtranslation_name ());
+    request->database_config_user()->setBible (filter::indonesian::ourtranslation ());
+  }
+
+  // Set the user chosen Bible as the current Bible.
+  if (request->post.count ("bibleselect")) {
+    string bibleselect = request->post ["bibleselect"];
+    request->database_config_user ()->setBible (bibleselect);
   }
 
   string page;
@@ -96,26 +103,28 @@ string read_index (void * webserver_request)
   
   Assets_View view;
   
-  if (request->query.count ("changebible")) {
-    string changebible = request->query ["changebible"];
-    if (changebible == "") {
-      Dialog_List dialog_list = Dialog_List ("index", translate("Select which Bible to read"), "", "");
-      vector <string> bibles = AccessBible::Bibles (request);
-      for (auto bible : bibles) {
-        dialog_list.add_row (bible, "changebible", bible);
-      }
-      page += dialog_list.run ();
-      return page;
-    } else {
-      request->database_config_user()->setBible (changebible);
-    }
-  }
-  
   // Get active Bible, and check read access to it.
+  // Or if the user have used query to preset the active Bible, get the preset Bible.
   // If needed, change Bible to one it has read access to.
+  // Set the chosen Bible on the option HTML tag.
   string bible = AccessBible::Clamp (request, request->database_config_user()->getBible ());
   if (request->query.count ("bible")) bible = AccessBible::Clamp (request, request->query ["bible"]);
+  string bible_html;
+  vector <string> bibles = AccessBible::Bibles (request);
+  for (auto selectable_bible : bibles) {
+    bible_html = Options_To_Select::add_selection (selectable_bible, selectable_bible, bible_html);
+  }
+  view.set_variable ("bibleoptags", Options_To_Select::mark_selected (bible, bible_html));
   view.set_variable ("bible", bible);
+
+  // In the Indonesian Cloud Free Simple version, its possible to get a specific book by query.
+  if (config_logic_indonesian_cloud_free_simple ()) {
+    if (request->query.count ("bbn")) {
+      int bible_book_number = convert_to_int (request->query ["bbn"]);
+      Ipc_Focus::set (request, bible_book_number, 1, 1);
+      Navigation_Passage::record_history (request, bible_book_number, 1, 1);
+    }
+  }
   
   // Store the active Bible in the page's javascript.
   view.set_variable ("navigationCode", Navigation_Passage::code (bible));
