@@ -28,6 +28,7 @@
 #include <locale/translate.h>
 #include <database/volatile.h>
 #include <database/config/general.h>
+#include <database/books.h>
 #include <access/bible.h>
 #include <ipc/focus.h>
 #include <search/logic.h>
@@ -72,14 +73,14 @@ string search_search2 (void * webserver_request)
     
     
     // Retrieve the search parameters from the volatile database.
-    string query = Database_Volatile::getValue (identifier, "query");
+    string query2 = Database_Volatile::getValue (identifier, "query");
     //bool casesensitive = convert_to_bool (Database_Volatile::getValue (identifier, "casesensitive"));
     bool plaintext = convert_to_bool (Database_Volatile::getValue (identifier, "plaintext"));
     
     
     // Get the Bible and passage for this identifier.
     Passage details = Passage::decode (hit);
-    string bible = details.bible;
+    string bible2 = details.bible;
     int book = details.book;
     int chapter = details.chapter;
     string verse = details.verse;
@@ -88,15 +89,15 @@ string search_search2 (void * webserver_request)
     // Get the plain text or USFM.
     string text;
     if (plaintext) {
-      text = search_logic_get_bible_verse_text (bible, book, chapter, convert_to_int (verse));
+      text = search_logic_get_bible_verse_text (bible2, book, chapter, convert_to_int (verse));
     } else {
-      text = search_logic_get_bible_verse_usfm (bible, book, chapter, convert_to_int (verse));
+      text = search_logic_get_bible_verse_usfm (bible2, book, chapter, convert_to_int (verse));
     }
     
     
     // Format it.
     string link = filter_passage_link_for_opening_editor_at (book, chapter, verse);
-    text =  filter_string_markup_words ({query}, text);
+    text =  filter_string_markup_words ({query2}, text);
     string output = "<div>" + link + " " + text + "</div>";
     
     
@@ -112,7 +113,7 @@ string search_search2 (void * webserver_request)
     // Get extra search parameters and store them all in the volatile database.
     bool casesensitive = (request->query ["c"] == "true");
     bool plaintext = (request->query ["p"] == "true");
-    bool currentbook = (request->query ["b"] == "true");
+    string books = request->query ["b"];
     string sharing = request->query ["s"];
     Database_Volatile::setValue (identifier, "query", query);
     Database_Volatile::setValue (identifier, "casesensitive", convert_to_string (casesensitive));
@@ -139,7 +140,7 @@ string search_search2 (void * webserver_request)
     
     
     // Deal with possible searching in the current book only.
-    if (currentbook) {
+    if (books == "currentbook") {
       int book = Ipc_Focus::getBook (request);
       vector <Passage> bookpassages;
       for (auto & passage : passages) {
@@ -150,6 +151,21 @@ string search_search2 (void * webserver_request)
       passages = bookpassages;
     }
     
+    
+    // Deal with possible searching in Old or New Testament only.
+    bool otbooks = (books == "otbooks");
+    bool ntbooks = (books == "ntbooks");
+    if (otbooks || ntbooks) {
+      vector <Passage> bookpassages;
+      for (auto & passage : passages) {
+        string type = Database_Books::getType (passage.book);
+        if (otbooks) if (type != "ot") continue;
+        if (ntbooks) if (type != "nt") continue;
+        bookpassages.push_back (passage);
+      }
+      passages = bookpassages;
+    }
+    
 
     // Deal with how to share the results.
     vector <string> hits;
