@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ string sendreceive_changes_up_to_date_text ()
 void sendreceive_changes ()
 {
   if (sendreceive_changes_watchdog) {
-    int time = filter_date_seconds_since_epoch ();
+    int time = filter::date::seconds_since_epoch ();
     if (time < (sendreceive_changes_watchdog + 900)) {
       Database_Logs::log (sendreceive_changes_text () + translate("Still busy"), Filter_Roles::translator ());
       return;
@@ -102,14 +102,14 @@ void sendreceive_changes ()
   
   
   // Set the correct user in the session: The sole user on the Client.
-  vector <string> users = request.database_users ()->getUsers ();
+  vector <string> users = request.database_users ()->get_users ();
   if (users.empty ()) {
     Database_Logs::log (translate("No user found"), Filter_Roles::translator ());
     send_receive_changes_done ();
     return;
   }
   string user = users [0];
-  request.session_logic ()->setUsername (user);
+  request.session_logic ()->set_username (user);
   string password = request.database_users ()->get_md5 (user);
   
   
@@ -193,8 +193,8 @@ void sendreceive_changes ()
     return;
   }
   {
-    vector <string> ids = filter_string_explode (response, '\n');
-    for (auto & id : ids) server_identifiers.push_back (convert_to_int (id));
+    vector <string> ids2 = filter_string_explode (response, '\n');
+    for (auto & id : ids2) server_identifiers.push_back (convert_to_int (id));
   }
 
   
@@ -275,5 +275,5 @@ void sendreceive_changes ()
 
 void sendreceive_changes_kick_watchdog ()
 {
-  sendreceive_changes_watchdog = filter_date_seconds_since_epoch ();
+  sendreceive_changes_watchdog = filter::date::seconds_since_epoch ();
 }
