@@ -117,7 +117,7 @@ void sendreceive_queue_sync (int minute, int second)
 
     if (sync_bibles || sync_rest) {
       // Store the most recent time that the sync action ran.
-      Database_Config_General::setLastSendReceive (filter_date_seconds_since_epoch ());
+      Database_Config_General::setLastSendReceive (filter::date::seconds_since_epoch ());
     }
   }
 }
@@ -137,15 +137,16 @@ bool sendreceive_sync_queued ()
 
 
 // Queues Paratext sync.
-void sendreceive_queue_paratext ()
+void sendreceive_queue_paratext (tasks::enums::paratext_sync method)
 {
 #ifdef HAVE_PARATEXT
   if (sendreceive_paratext_queued ()) {
     Database_Logs::log ("About to start synchronizing with Paratext");
   } else {
-    tasks_logic_queue (SYNCPARATEXT);
+    tasks_logic_queue (SYNCPARATEXT, { to_string(static_cast<int>(method)) });
   }
 #endif
+  (void) method;
 }
 
 
@@ -193,7 +194,7 @@ void sendreceive_queue_startup ()
   }
   
   // When the current time is past the next time it is supposed to sync, start the sync.
-  if (filter_date_seconds_since_epoch () >= next_second) {
+  if (filter::date::seconds_since_epoch () >= next_second) {
     sendreceive_queue_sync (-1, 0);
   }
 }
