@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -34,11 +34,8 @@ string sendreceive_tag ()
 }
 
 
-void sendreceive_sendreceive (string bible)
+void sendreceive_sendreceive ([[maybe_unused]] string bible)
 {
-#ifdef HAVE_CLIENT
-  (void) bible;
-#endif
 #ifdef HAVE_CLOUD
   // Check on Bible.
   if (bible.empty ()) {
@@ -64,11 +61,14 @@ void sendreceive_sendreceive (string bible)
   }
   
   
+  // Log what this is going to do.
   if (read_from_git) {
     Database_Logs::log (sendreceive_tag () + sendreceive_sendreceive_sendreceive_text () + bible, Filter_Roles::translator ());
   } else {
     Database_Logs::log (sendreceive_tag () + sendreceive_sendreceive_send_text () + bible, Filter_Roles::translator ());
   }
+  
+  
   Webserver_Request request;
   bool success = true;
   string error;
@@ -90,7 +90,7 @@ void sendreceive_sendreceive (string bible)
   // Set a flag indicating whether there are local changes available.
   bool localchanges = false;
   if (success) {
-    vector <string> lines = filter_git_status (directory);
+    vector <string> lines = filter_git_status (directory, true);
     for (auto & line : lines) {
       Passage passage = filter_git_get_passage (line);
       if (passage.book) {
@@ -101,7 +101,7 @@ void sendreceive_sendreceive (string bible)
   }
   
   
-  // In case of local changes, add / remove the updated data to the index.
+  // In case of local changes, add/remove the updated data to/from the index.
   if (success && localchanges) {
     success = filter_git_add_remove_all (directory, error);
     if (!success) {
@@ -149,8 +149,9 @@ void sendreceive_sendreceive (string bible)
       filter_git_resolve_conflicts (directory, paths_resolved_conflicts, error);
       if (!error.empty ()) Database_Logs::log (error, Filter_Roles::translator ());
       vector <string> messages;
-      string error;
-      filter_git_commit (directory, "", translate ("Bibledit resolved the conflicts"), messages, error);
+      string tmp_error;
+      string no_user {};
+      filter_git_commit (directory, no_user, translate ("Bibledit resolved the conflicts"), messages, tmp_error);
       for (auto & msg : messages) Database_Logs::log (sendreceive_tag () + "conflict resolution: " + msg, Filter_Roles::translator ());
       // The above "git pull" operation failed due to the conflict(s).
       // Since the conflicts have now been resolved, set "success" to true again.
