@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ void sendreceive_settings_done ()
 void sendreceive_settings ()
 {
   if (sendreceive_settings_watchdog) {
-    int time = filter_date_seconds_since_epoch ();
+    int time = filter::date::seconds_since_epoch ();
     if (time < (sendreceive_settings_watchdog + 900)) {
       Database_Logs::log ("Settings: " + translate("Still busy"), Filter_Roles::translator ());
       return;
@@ -91,14 +91,14 @@ void sendreceive_settings ()
   }
 
   // Set the correct user in the session: The sole user on the Client.
-  vector <string> users = request.database_users ()->getUsers ();
+  vector <string> users = request.database_users ()->get_users ();
   if (users.empty ()) {
     Database_Logs::log (translate("No user found"), Filter_Roles::translator ());
     sendreceive_settings_done ();
     return;
   }
   string user = users [0];
-  request.session_logic ()->setUsername (user);
+  request.session_logic ()->set_username (user);
   
   string address = Database_Config_General::getServerAddress ();
   int port = Database_Config_General::getServerPort ();
@@ -262,6 +262,6 @@ void sendreceive_settings ()
 
 void sendreceive_settings_kick_watchdog ()
 {
-  sendreceive_settings_watchdog = filter_date_seconds_since_epoch ();
+  sendreceive_settings_watchdog = filter::date::seconds_since_epoch ();
 }
 
