@@ -72,22 +72,22 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
   if (request->query.count ("previoussprint")) {
     int month = request->database_config_user()->getSprintMonth ();
     int year = request->database_config_user()->getSprintYear ();
-    filter_date_get_previous_month (month, year);
+    filter::date::get_previous_month (month, year);
     request->database_config_user()->setSprintMonth (month);
     request->database_config_user()->setSprintYear (year);
   }
   
   
   if (request->query.count ("currentprint")) {
-    request->database_config_user()->setSprintMonth (filter_date_numerical_month (filter_date_seconds_since_epoch ()));
-    request->database_config_user()->setSprintYear (filter_date_numerical_year (filter_date_seconds_since_epoch ()));
+    request->database_config_user()->setSprintMonth (filter::date::numerical_month (filter::date::seconds_since_epoch ()));
+    request->database_config_user()->setSprintYear (filter::date::numerical_year (filter::date::seconds_since_epoch ()));
   }
   
   
   if (request->query.count ("nextsprint")) {
     int month = request->database_config_user()->getSprintMonth ();
     int year = request->database_config_user()->getSprintYear ();
-    filter_date_get_next_month (month, year);
+    filter::date::get_next_month (month, year);
     request->database_config_user()->setSprintMonth (month);
     request->database_config_user()->setSprintYear (year);
   }
@@ -119,8 +119,8 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
         float category_percentage = 100 / category_count;
         int percentage;
         bool on = (checked == "true");
-        if (on) percentage = round ((box + 1) * category_percentage);
-        else percentage = round (box * category_percentage);
+        if (on) percentage = static_cast <int> (round ((box + 1) * category_percentage));
+        else percentage = static_cast <int> (round (box * category_percentage));
         database_sprint.updateComplete (identifier, percentage);
       }
     }
@@ -138,20 +138,20 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
   
   
   if (request->query.count ("mail")) {
-    int year = request->database_config_user()->getSprintYear ();
-    int month = request->database_config_user()->getSprintMonth ();
-    sprint_burndown (bible, year, month);
+    int mail_year = request->database_config_user()->getSprintYear ();
+    int mail_month = request->database_config_user()->getSprintMonth ();
+    sprint_burndown (bible, mail_year, mail_month);
     view.set_variable ("success", translate("The information was mailed to the subscribers"));
   }
   
   
   if (request->query.count ("bible")) {
     bible = request->query ["bible"];
-    if (bible == "") {
+    if (bible.empty()) {
       Dialog_List dialog_list = Dialog_List ("index", translate("Select which Bible to display the Sprint for"), "", "");
       vector <string> bibles = AccessBible::Bibles (request);
-      for (auto & bible : bibles) {
-        dialog_list.add_row (bible, "bible", bible);
+      for (auto & selection_bible : bibles) {
+        dialog_list.add_row (selection_bible, "bible", selection_bible);
       }
       page += dialog_list.run ();
       return page;
@@ -174,7 +174,7 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
   
   
   if (request->query.count ("moveback")) {
-    filter_date_get_previous_month (month, year);
+    filter::date::get_previous_month (month, year);
     database_sprint.updateMonthYear (id, month, year);
     view.set_variable ("success", translate("The task was moved to the previous sprint"));
     request->database_config_user()->setSprintMonth (month);
@@ -183,7 +183,7 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
                         
                         
   if (request->query.count ("moveforward")) {
-    filter_date_get_next_month (month, year);
+    filter::date::get_next_month (month, year);
     database_sprint.updateMonthYear (id, month, year);
     view.set_variable ("success", translate("The task was moved to the next sprint"));
     request->database_config_user()->setSprintMonth (month);
@@ -220,22 +220,22 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
   
   
   string tasks;
-  vector <int> vtasks = database_sprint.getTasks (bible, year, month);
-  for (auto & id : vtasks) {
-    string title = escape_special_xml_characters (database_sprint.getTitle (id));
-    int percentage = database_sprint.getComplete (id);
-    tasks.append ("<tr id=\"a" + convert_to_string (id) + "\">\n");
-    tasks.append ("<td><a href=\"?id=" + convert_to_string (id) + "&remove=\">" + emoji_wastebasket () + "</a></td>\n");
+  vector <int> v_tasks = database_sprint.getTasks (bible, year, month);
+  for (auto & task_id : v_tasks) {
+    string title = escape_special_xml_characters (database_sprint.getTitle (task_id));
+    int percentage = database_sprint.getComplete (task_id);
+    tasks.append ("<tr id=\"a" + convert_to_string (task_id) + "\">\n");
+    tasks.append ("<td><a href=\"?id=" + convert_to_string (task_id) + "&remove=\">" + emoji_wastebasket () + "</a></td>\n");
     tasks.append ("<td></td>\n");
-    tasks.append ("<td><a href=\"?id=" + convert_to_string (id) + "&moveback=\"> « </a></td>\n");
+    tasks.append ("<td><a href=\"?id=" + convert_to_string (task_id) + "&moveback=\"> « </a></td>\n");
     tasks.append ("<td>" + title + "</td>\n");
     size_t category_count = vcategories.size();
     float category_percentage = 100 / category_count;
     for (size_t i2 = 0; i2 < vcategories.size (); i2++) {
-      int high = round ((i2 + 1) * category_percentage);
+      int high = static_cast <int> (round ((i2 + 1) * category_percentage));
       tasks.append ("<td>\n");
       tasks.append ("<input type=\"checkbox\" id=\"task");
-      tasks.append (convert_to_string (id));
+      tasks.append (convert_to_string (task_id));
       tasks.append ("box");
       tasks.append (convert_to_string (i2));
       tasks.append ("\"");
@@ -247,7 +247,7 @@ string sprint_index ([[maybe_unused]] void * webserver_request)
       
       tasks.append ("</td>\n");
     }
-    tasks.append ("<td><a href=\"?id=" + convert_to_string (id) + "&moveforward=\"> » </a></td>\n");
+    tasks.append ("<td><a href=\"?id=" + convert_to_string (task_id) + "&moveforward=\"> » </a></td>\n");
     tasks.append ("</tr>\n");
   }
   view.set_variable ("tasks", tasks);
