@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2020 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ string sync_bibles_receive_chapter (Webserver_Request * request, string & bible,
   
   
   // Check whether the user has write-access to the Bible book.
-  if (!access_bible_book_write (request, username, bible, book)) {
+  if (!AccessBible::BookWrite (request, username, bible, book)) {
     string message = "User " + username + " does not have write access to Bible " + bible;
     Database_Logs::log (message, Filter_Roles::manager ());
     // The Cloud will email the user with details about the issue.
@@ -84,14 +84,15 @@ string sync_bibles_receive_chapter (Webserver_Request * request, string & bible,
   string serverusfm = request->database_bibles()->getChapter (bible, book, chapter);
   
   
-  // There have been cases that USFM with double spaces in them caused the server to add one more space.
-  // Thus the number of added spaces kept increasing each time the client sent a chapter to the server.
-  // Remove all the double spaces.
-  for (int i = 0; i < 10; i++) {
-    if (oldusfm.find ("  ")) oldusfm = filter_string_str_replace ("  ", " ", oldusfm);
-    if (newusfm.find ("  ")) newusfm = filter_string_str_replace ("  ", " ", newusfm);
-    if (serverusfm.find ("  ")) serverusfm = filter_string_str_replace ("  ", " ", serverusfm);
-  }
+  // There have been cases like this:
+  // The Bibledit client sends USFM with double spaces in them to the Cloud.
+  // Then the Cloud would add one one more space.
+  // If this chapter was edited and sent multiple times,
+  // the result would be that each time extra spaces kept being added.
+  // Solution / fix to this:
+  // Remove all the double spaces from the USFM to be saved.
+  // (Keep the reference USFM as it is: Goal: Detect the differences.
+  newusfm = filter_string_collapse_whitespace (newusfm);
   
 
   // Gather data for recording the changes made by the user, for the change notifications.
@@ -114,6 +115,7 @@ string sync_bibles_receive_chapter (Webserver_Request * request, string & bible,
     // Update the server with the new chapter data.
     bible_logic_store_chapter (bible, book, chapter, mergedusfm);
     // Check on the merge.
+    filter_merge_add_book_chapter (conflicts, book, chapter);
     bible_logic_client_receive_merge_mail (bible, book, chapter, username, oldusfm, newusfm, mergedusfm);
     // Log the merge in the journal, for possible trouble shooting.
     bible_logic_log_merge (username, bible, book, chapter, oldusfm, newusfm, serverusfm, mergedusfm);
@@ -145,7 +147,7 @@ string sync_bibles_receive_chapter (Webserver_Request * request, string & bible,
 
 string sync_bibles (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   Sync_Logic sync_logic = Sync_Logic (webserver_request);
 
   if (!sync_logic.security_okay ()) {
@@ -174,7 +176,7 @@ string sync_bibles (void * webserver_request)
       // calculate the checksum of all chapters in those Bibles,
       // and returns this checksum to the client.
       string username = request->session_logic ()->currentUser ();
-      vector <string> bibles = access_bible_bibles (request, username);
+      vector <string> bibles = AccessBible::Bibles (request, username);
       string server_checksum = Checksum_Logic::getBibles (request, bibles);
       return server_checksum;
     }
@@ -183,7 +185,7 @@ string sync_bibles (void * webserver_request)
       // The server reads the credentials from the client's user,
       // and responds with a list of Bibles this user has access to.
       string username = request->session_logic ()->currentUser ();
-      vector <string> bibles = access_bible_bibles (request, username);
+      vector <string> bibles = AccessBible::Bibles (request, username);
       string checksum = Checksum_Logic::get (bibles);
       string s_bibles = filter_string_implode (bibles, "\n");
       return checksum + "\n" + s_bibles;
@@ -198,7 +200,7 @@ string sync_bibles (void * webserver_request)
       // The server responds with a checksum and then the list of books in the Bible.
       vector <int> server_books = request->database_bibles()->getBooks (bible);
       vector <string> v_server_books;
-      for (auto book : server_books) v_server_books.push_back (convert_to_string (book));
+      for (auto server_book : server_books) v_server_books.push_back (convert_to_string (server_book));
       string s_server_books = filter_string_implode (v_server_books, "\n");
       string checksum = Checksum_Logic::get (v_server_books);
       return checksum + "\n" + s_server_books;
@@ -213,7 +215,7 @@ string sync_bibles (void * webserver_request)
       // The server responds with the list of books in the Bible book.
       vector <int> server_chapters = request->database_bibles()->getChapters (bible, book);
       vector <string> v_server_chapters;
-      for (auto & chapter : server_chapters) v_server_chapters.push_back (convert_to_string (chapter));
+      for (auto & server_chapter : server_chapters) v_server_chapters.push_back (convert_to_string (server_chapter));
       string s_server_chapters = filter_string_implode (v_server_chapters, "\n");
       string checksum = Checksum_Logic::get (v_server_chapters);
       return checksum + "\n" + s_server_chapters;
