@@ -94,7 +94,7 @@ string sync_notes (void * webserver_request)
       return "";
     }
   }
-  request->session_logic ()->setUsername (user);
+  request->session_logic ()->set_username (user);
   
   
   // Note lower and upper limits.
@@ -125,11 +125,11 @@ string sync_notes (void * webserver_request)
       vector <string> bibles = AccessBible::Bibles (webserver_request, user);
       vector <int> identifiers = database_notes.get_notes_in_range_for_bibles (lowId, highId, bibles, false);
       string response;
-      for (auto identifier : identifiers) {
+      for (auto id : identifiers) {
         if (!response.empty ()) response.append ("\n");
-        response.append (convert_to_string (identifier));
+        response.append (convert_to_string (id));
         response.append ("\n");
-        response.append (database_notes.get_checksum (identifier));
+        response.append (database_notes.get_checksum (id));
       }
       return response;
     }
