@@ -160,15 +160,15 @@ void system_logic_import_bibles_file (string tarball)
     // Get details about the USFM to import.
     string stylesheet = styles_logic_standard_sheet ();
     vector <BookChapterData> book_chapter_text = usfm_import (data, stylesheet);
-    for (auto & data : book_chapter_text) {
-      if (data.book > 0) {
+    for (auto & book_chapter_data : book_chapter_text) {
+      if (book_chapter_data.book > 0) {
         // Store the data and log it.
         // This does not trigger the client to send it to the Cloud.
         // Reason is that the Cloud is authoritative,
         // so importing outdated Bibles would not affect the authoritative copy in the Cloud.
-        database_bibles.storeChapter (bible, data.book, data.chapter, data.data);
-        string bookname = Database_Books::getEnglishFromId (data.book);
-        Database_Logs::log ("Imported " + bible + " " + bookname + " " + convert_to_string (data.chapter));
+        database_bibles.storeChapter (bible, book_chapter_data.book, book_chapter_data.chapter, book_chapter_data.data);
+        string bookname = Database_Books::getEnglishFromId (book_chapter_data.book);
+        Database_Logs::log ("Imported " + bible + " " + bookname + " " + convert_to_string (book_chapter_data.chapter));
       } else {
         // Import error.
         Database_Logs::log ("Could not import this file: " + file);
@@ -342,7 +342,7 @@ void system_logic_produce_resources_file (int jobid)
 
   
   // Progress bar data: How many tarballs to create.
-  size_t tarball_count = 1 + single_resources.size ();
+  int tarball_count = static_cast<int>(1 + single_resources.size ());
   int tarball_counter = 0;
 
   
@@ -362,11 +362,11 @@ void system_logic_produce_resources_file (int jobid)
   for (auto element : single_resources) {
     tarball_counter++;
     string resource_name = element.first;
-    vector <string> resources = element.second;
+    vector <string> v_resources = element.second;
     database_jobs.set_percentage (jobid, 100 * tarball_counter / tarball_count);
     database_jobs.set_progress (jobid, resource_name);
-    string tarball = filter_url_create_root_path ({system_logic_resources_file_name (resource_name)});
-    string error = filter_archive_microtar_pack (tarball, directory, resources);
+    string resource_tarball = filter_url_create_root_path ({system_logic_resources_file_name (resource_name)});
+    error += filter_archive_microtar_pack (resource_tarball, directory, v_resources);
   }
   
   
@@ -495,7 +495,7 @@ void system_logic_indonesian_free_deletion ([[maybe_unused]] string username,
   }
   
   Database_Bibles database_bibles;
-  string bible = filter_indonesian_terjemahanku_mytranslation_name (username);
+  string bible = filter::indonesian::mytranslation (username);
   vector <int> books = database_bibles.getBooks (bible);
   for (auto book : books) {
     vector <int> chapters = database_bibles.getChapters (bible, book);
@@ -557,7 +557,7 @@ void system_logic_indonesian_free_expiration ()
     int level = database_users.get_level(user);
     if (level != Filter_Roles::consultant()) continue;
     // Expire this account after 30 days.
-    int account_creation_time = filter_date_seconds_since_epoch();
+    int account_creation_time = filter::date::seconds_since_epoch();
     {
       vector <string> lines = Database_Config_General::getAccountCreationTimes ();
       for (auto line : lines) {
@@ -567,7 +567,7 @@ void system_logic_indonesian_free_expiration ()
         if (user == bits[1]) account_creation_time = seconds;
       }
     }
-    int seconds = filter_date_seconds_since_epoch() - account_creation_time;
+    int seconds = filter::date::seconds_since_epoch() - account_creation_time;
     int days = seconds / (3600 * 24);
     if (days <= 30) continue;
     // Get details of this user.
@@ -586,14 +586,14 @@ void system_logic_indonesian_free_expiration ()
   vector<string> valid_indonesian_bibles;
   users = database_users.get_users();
   for (auto user : users) {
-    string bible = filter_indonesian_terjemahanku_mytranslation_name (user);
+    string bible = filter::indonesian::mytranslation (user);
     valid_indonesian_bibles.push_back(bible);
   }
   Database_Confirm database_confirm;
   vector <int> ids = database_confirm.get_ids();
   for (auto id : ids) {
-    string username = database_confirm.get_username(id);
-    string bible = filter_indonesian_terjemahanku_mytranslation_name (username);
+    string username = database_confirm.get_username(static_cast<unsigned int>(id));
+    string bible = filter::indonesian::mytranslation (username);
     valid_indonesian_bibles.push_back(bible);
   }
   // Get all available Bibles at the system.
@@ -603,7 +603,7 @@ void system_logic_indonesian_free_expiration ()
   vector <string> bibles_to_delete = filter_string_array_diff(bibles, valid_indonesian_bibles);
   for (auto bible : bibles_to_delete) {
     // Skip the shared Indonesian Bible.
-    if (bible == filter_indonesian_alkitabkita_ourtranslation_name()) continue;
+    if (bible == filter::indonesian::ourtranslation()) continue;
     // Delete this unassociated Bible.
     Database_Logs::log("Deleting Bible not associated with any account: " + bible);
     database_bibles.deleteBible(bible);
