@@ -1,5 +1,5 @@
 /*
-Copyright (©) 2003-2020 Teus Benschop.
+Copyright (©) 2003-2022 Teus Benschop.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <webserver/io.h>
 #include <filter/string.h>
 #include <filter/url.h>
+#include <filter/date.h>
 #include <mbedtls/entropy.h>
 #include <mbedtls/ctr_drbg.h>
 #include <mbedtls/certs.h>
@@ -57,7 +58,7 @@ int get_line (int sock, char *buf, int size)
   char character = '\0';
   int n = 0;
   while ((i < size - 1) && (character != '\n')) {
-    n = recv (sock, &character, 1, 0);
+    n = (int) recv (sock, &character, 1, 0);
     if (n > 0) {
       if (character == '\r') {
         /*
@@ -66,7 +67,7 @@ int get_line (int sock, char *buf, int size)
         // So it's safe to throw the \r away, as we're sure the \n follows.
         continue;
         */
-        n = recv (sock, &character, 1, MSG_PEEK);
+        n = (int) recv (sock, &character, 1, MSG_PEEK);
         if ((n > 0) && (character == '\n')) {
           recv (sock, &character, 1, 0);
         } else {
@@ -134,7 +135,7 @@ void webserver_process_request (int connfd, string clientaddress)
           bool done_reading = false;
           int total_bytes_read = 0;
           do {
-            bytes_read = recv(connfd, buffer, BUFFERSIZE, 0);
+            bytes_read = (int)recv(connfd, buffer, BUFFERSIZE, 0);
             for (int i = 0; i < bytes_read; i++) {
               postdata += buffer [i];
             }
@@ -177,19 +178,18 @@ void webserver_process_request (int connfd, string clientaddress)
             open
 #endif
             (request.stream_file.c_str(), O_RDONLY);
-            unsigned char buffer [1024];
+            unsigned char streambuffer [1024];
             int bytecount;
             do {
-              bytecount =
+              bytecount = (int)
 #ifdef HAVE_WINDOWS
               _read
 #else
               read
 #endif
-              (filefd, buffer, 1024);
+              (filefd, streambuffer, 1024);
               if (bytecount > 0) {
-                int sendbytes = send (connfd, (const char *)buffer, bytecount, 0);
-                (void) sendbytes;
+                [[maybe_unused]] auto sendbytes = send (connfd, (const char *)streambuffer, static_cast<size_t>(bytecount), 0);
               }
             }
             while (bytecount > 0);
@@ -554,7 +554,7 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
         if (ret == 0) header_parsed = false; // 0: EOF
         if (ret < 0) connection_healthy = false;
         if (connection_healthy && header_parsed) {
-          char c = buffer [0];
+          char c = static_cast <char> (buffer [0]);
           // The request contains a carriage return (\r) and a new line feed (\n).
           // The traditional order of this is \r\n.
           // Therefore when a \r is encountered, just disregard it.
@@ -588,7 +588,7 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
           if (ret == 0) done_reading = true; // 0: EOF
           if (ret < 0) connection_healthy = false;
           if (connection_healthy && !done_reading) {
-            char c = buffer [0];
+            char c = static_cast <char> (buffer [0]);
             postdata += c;
             total_bytes_read ++;
           }
@@ -624,7 +624,7 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
         ret = mbedtls_ssl_write (&ssl, buf, len);
         if (ret > 0) {
           buf += ret;
-          len -= ret;
+          len -= static_cast <size_t> (ret);
         } else {
           // When it returns MBEDTLS_ERR_SSL_WANT_WRITE/READ,
           // it must be called later with the *same* arguments,
@@ -650,16 +650,16 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
         unsigned char buffer [1024];
         int bytecount;
         do {
-          bytecount =
+          bytecount = (int)
 #ifdef HAVE_WINDOWS
           _read
 #else
           read
 #endif
           (filefd, buffer, 1024);
-          int len = bytecount;
-          const unsigned char * buf = (const unsigned char *) &buffer;
-          while (connection_healthy && (len > 0)) {
+          int remaining_length = bytecount;
+          const unsigned char * buffer_ptr = (const unsigned char *) &buffer;
+          while (connection_healthy && (remaining_length > 0)) {
             // Function
             // int ret = mbedtls_ssl_write (&ssl, buf, len)
             // will do partial writes in some cases.
@@ -667,10 +667,10 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
             // the function must be called again with updated arguments:
             // buf + ret, len - ret
             // until it returns a value equal to the last 'len' argument.
-            ret = mbedtls_ssl_write (&ssl, buf, len);
+            ret = mbedtls_ssl_write (&ssl, buffer_ptr, static_cast<size_t>(remaining_length));
             if (ret > 0) {
-              buf += ret;
-              len -= ret;
+              buffer_ptr += ret;
+              remaining_length -= ret;
             } else {
               // When it returns MBEDTLS_ERR_SSL_WANT_WRITE/READ,
               // it must be called later with the *same* arguments,
@@ -698,6 +698,7 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
           if (ret == MBEDTLS_ERR_SSL_WANT_WRITE) continue;
           filter_url_display_mbed_tls_error (ret, NULL, true);
           connection_healthy = false;
+          if (connection_healthy) {}; // Suppress static analyzer warning about unused code.
           break;
         }
       }
@@ -725,11 +726,16 @@ void secure_webserver_process_request (mbedtls_ssl_config * conf, mbedtls_net_co
 
 void https_server ()
 {
-#ifdef RUN_SECURE_SERVER
   // On clients, don't run the secure web server.
   // It is not possible to get a https certificate for https://localhost anyway.
   // Not running this secure server saves valuable system resources on low power devices.
+#ifdef RUN_SECURE_SERVER
 
+  // The https network port to listen on.
+  // Port "0..9" means: Don't run the secure web server.
+  string network_port = config_logic_https_network_port ();
+  if (network_port.length() <= 1) return;
+  
   // File descriptor for the listener.
   mbedtls_net_context listen_fd;
   mbedtls_net_init (&listen_fd);
@@ -782,7 +788,7 @@ void https_server ()
   }
   
   // Setup the listening TCP socket.
-  ret = mbedtls_net_bind (&listen_fd, NULL, convert_to_string (config_logic_https_network_port ()).c_str (), MBEDTLS_NET_PROTO_TCP);
+  ret = mbedtls_net_bind (&listen_fd, NULL, network_port.c_str (), MBEDTLS_NET_PROTO_TCP);
   if (ret != 0) {
     filter_url_display_mbed_tls_error (ret, NULL, true);
     return;
