@@ -1,5 +1,5 @@
 /*
- Copyright (©) 2003-2019 Teus Benschop.
+ Copyright (©) 2003-2022 Teus Benschop.
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -22,8 +22,9 @@
 #include <filter/url.h>
 #include <webserver/request.h>
 #include <edit/index.h>
-#include <editone/index.h>
-#include <editverse/index.h>
+#include <edit/index.h>
+#include <redirect/index.h>
+#include <editone2/index.h>
 #include <editusfm/index.h>
 #include <search/index.h>
 #include <resource/index.h>
@@ -32,6 +33,8 @@
 #include <sync/logic.h>
 #include <locale/translate.h>
 #include <database/logs.h>
+#include <database/cache.h>
+#include <read/index.h>
 
 
 vector <string> workspace_get_default_names ()
@@ -39,6 +42,11 @@ vector <string> workspace_get_default_names ()
   // Any of the names below should not contain commas,
   // because the sorting mechanism takes the comma as a separator,
   // so if commas are in a name, the sorting no longer works.
+  if (config_logic_indonesian_cloud_free_simple ()) {
+    return {
+      "Baca dan Teliti"
+    };
+  };
   return {
     translate ("Editor and Resources"),
     translate ("Editor and Notes"),
@@ -54,34 +62,41 @@ map <int, string> workspace_get_default_urls (int id)
   map <int, string> urls;
   switch (id) {
     case 1:
-      urls [0] = editone_index_url ();
+      urls [0] = editone2_index_url ();
       urls [5] = resource_index_url ();
       break;
     case 2:
-      urls [0] = editone_index_url ();
+      urls [0] = editone2_index_url ();
       urls [1] = notes_index_url ();
       break;
     case 3:
       urls [0] = resource_index_url ();
-      urls [1] = editone_index_url ();
+      urls [1] = editone2_index_url ();
       urls [2] = notes_index_url ();
       break;
     case 4:
-      urls [0] = editone_index_url ();
+      urls [0] = editone2_index_url ();
       urls [1] = consistency_index_url ();
       break;
     case 5:
       urls [0] = resource_index_url ();
-      urls [1] = editone_index_url ();
-      urls [5] = editverse_index_url ();
+      urls [1] = editone2_index_url ();
+      urls [5] = editusfm_index_url ();
       break;
     default:
-      urls [0] = editone_index_url ();
+      urls [0] = editone2_index_url ();
       urls [1] = resource_index_url ();
       urls [2] = notes_index_url ();
       urls [3] = search_index_url ();
       break;
   }
+  if (config_logic_indonesian_cloud_free_simple ()) {
+    urls [0] = read_index_url ();
+    urls [1] = resource_index_url ();
+    urls [2] = "";
+    urls [3] = "";
+    urls [5] = "";
+  }
   return urls;
 }
 
@@ -119,6 +134,13 @@ map <int, string> workspace_get_default_widths (int id)
       widths [3] = "1";
       break;
   }
+  if (config_logic_indonesian_cloud_free_simple ()) {
+    widths [0] = "1";
+    widths [1] = "1";
+    widths [2] = "0";
+    widths [3] = "0";
+    widths [5] = "0";
+  }
   return widths;
 }
 
@@ -144,13 +166,17 @@ map <int, string> workspace_get_default_heights (int id)
       heights [0] = "1";
       break;
   }
+  if (config_logic_indonesian_cloud_free_simple ()) {
+    heights [0] = "1";
+    heights [1] = "0";
+  }
   return heights;
 }
 
 
 void workspace_create_defaults (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   // Save current active workspace.
   string workspace = request->database_config_user()->getActiveWorkspace ();
@@ -159,7 +185,7 @@ void workspace_create_defaults (void * webserver_request)
   vector <string> names = workspace_get_default_names ();
   for (unsigned int i = 0; i < names.size (); i++) {
     request->database_config_user()->setActiveWorkspace (names [i]);
-    int bench = i + 1;
+    int bench = static_cast<int>(i + 1);
     workspace_set_urls (request, workspace_get_default_urls (bench));
     workspace_set_widths (request, workspace_get_default_widths (bench));
     workspace_set_heights (request, workspace_get_default_heights (bench));
@@ -172,8 +198,10 @@ void workspace_create_defaults (void * webserver_request)
 
 string workspace_get_active_name (void * webserver_request)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
+
   string workspace = request->database_config_user()->getActiveWorkspace ();
+
   if (workspace.empty ()) {
     workspace = workspace_get_default_name ();
   }
@@ -187,10 +215,8 @@ string workspace_process_units (string length)
   // If a size factor is found, great, otherwise default to 1
   if (length == convert_to_string (convert_to_int (length))) {
     return length;
-  } else {
-    string length = "1";
-    return length;
   }
+  return "1";
 }
 
 
@@ -203,7 +229,7 @@ string workspace_process_units (string length)
 void workspace_set_values (void * webserver_request, int selector, const map <int, string> & values)
 {
   // Store values locally, and for a client, store them also for sending to the server.
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   string workspace = workspace_get_active_name (request);
   string rawvalue;
   if (selector == URLS) rawvalue = request->database_config_user()->getWorkspaceURLs ();
@@ -266,14 +292,14 @@ void workspace_set_heights (void * webserver_request, const map <int, string> &
 
 void workspace_set_entire_width (void * webserver_request, string value)
 {
-  map <int, string> values = {make_pair (0, value)};
+  map <int, string> values = {pair (0, value)};
   workspace_set_values (webserver_request, ENTIREWIDTH, values);
 }
 
 
 map <int, string> workspace_get_values (void * webserver_request, int selector, bool use)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   map <int, string> values;
   
@@ -359,8 +385,11 @@ map <int, string> workspace_get_heights (void * webserver_request)
 string workspace_get_entire_width (void * webserver_request)
 {
   map <int, string> values = workspace_get_values (webserver_request, ENTIREWIDTH, false);
-  for (auto & element : values) return element.second;
-  return "";
+  string width;
+  for (auto & element : values) {
+    width = element.second;
+  }
+  return width;;
 }
 
 
@@ -368,7 +397,7 @@ string workspace_get_entire_width (void * webserver_request)
 // If $add_default, if there's no workspaces, it adds a default one.
 vector <string> workspace_get_names (void * webserver_request, bool add_default)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   vector <string> workspaces;
   // The names and the order of the workspaces is taken from the URLs.
   string rawvalue = request->database_config_user()->getWorkspaceURLs ();
@@ -390,7 +419,7 @@ vector <string> workspace_get_names (void * webserver_request, bool add_default)
 
 void workspace_delete (void * webserver_request, string workspace)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   string rawvalue;
   vector <string> currentlines;
@@ -437,7 +466,7 @@ void workspace_reorder (void * webserver_request, const vector <string> & worksp
   // The order of the workspaces is taken from the URLs.
   // Widths and heights are not considered for the order.
   
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   // Retrieve the old order of the workspaces, plus their details.
   string rawvalue = request->database_config_user()->getWorkspaceURLs ();
@@ -473,7 +502,7 @@ void workspace_reorder (void * webserver_request, const vector <string> & worksp
 // Copy workspace $source to $destination
 void workspace_copy (void * webserver_request, string source, string destination)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   // Save current active workspace.
   string active_workspace = request->database_config_user()->getActiveWorkspace ();
@@ -496,22 +525,20 @@ void workspace_copy (void * webserver_request, string source, string destination
 
 
 // Store updated workspace settings for sending to the cloud.
-void workspace_cache_for_cloud (void * webserver_request, bool urls, bool widths, bool heights)
+void workspace_cache_for_cloud ([[maybe_unused]] void * webserver_request,
+                                [[maybe_unused]] bool urls,
+                                [[maybe_unused]] bool widths,
+                                [[maybe_unused]] bool heights)
 {
 #ifdef HAVE_CLIENT
   // For a client, store the setting for sending to the server.
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
   if (urls)
     request->database_config_user()->addUpdatedSetting (Sync_Logic::settings_send_workspace_urls);
   if (widths)
     request->database_config_user()->addUpdatedSetting (Sync_Logic::settings_send_workspace_widths);
   if (heights)
     request->database_config_user()->addUpdatedSetting (Sync_Logic::settings_send_workspace_heights);
-#else
-  (void) webserver_request;
-  (void) urls;
-  (void) widths;
-  (void) heights;
 #endif
 }
 
@@ -525,7 +552,7 @@ string workspace_get_default_name ()
 // Send the named $workspace to a $user name.
 void workspace_send (void * webserver_request, string workspace, string user)
 {
-  Webserver_Request * request = (Webserver_Request *) webserver_request;
+  Webserver_Request * request = static_cast<Webserver_Request *>(webserver_request);
 
   // Save current active workspace.
   string active_workspace = request->database_config_user()->getActiveWorkspace ();
@@ -542,7 +569,7 @@ void workspace_send (void * webserver_request, string workspace, string user)
 
   // New webserver request object for the destination user.
   Webserver_Request destination_request;
-  destination_request.session_logic ()->setUsername (user);
+  destination_request.session_logic ()->set_username (user);
   
   // Save workspace for destination user.
   active_workspace = destination_request.database_config_user()->getActiveWorkspace ();
@@ -559,19 +586,19 @@ void workspace_send (void * webserver_request, string workspace, string user)
 }
 
 
-string workspace_query_key_readonly ()
-{
-  return "readonly";
-}
-
-
 // https://github.com/bibledit/cloud/issues/342
 // One workspace can only have one editable Bible editor.
 // The first Bible editor remains editable.
 // Any subsequent Bible editors will be set read-only.
 // This is related to focused caret jumping that leads to confusion.
-void workspace_add_bible_readonly (map <int, string> & urls)
+// See also issue https://github.com/bibledit/cloud/issues/391 for
+// why the editor numbers are passed to the iframes.
+// The reason is that each editor's Javascript can determine
+// which Bible editor number it is.
+// It can then decide to make the editor read-only.
+map <int, int> workspace_add_bible_editor_number (map <int, string> & urls)
 {
+  map <int, int> editor_numbers;
   int bible_editor_count = 0;
   for (auto & element : urls) {
     bool is_bible_editor = false;
@@ -579,14 +606,16 @@ void workspace_add_bible_readonly (map <int, string> & urls)
     if (url.empty()) continue;
     if (url.find (edit_index_url ()) != string::npos) is_bible_editor = true;
     if (url.find (editone_index_url ()) != string::npos) is_bible_editor = true;
+    if (url.find (editone2_index_url ()) != string::npos) is_bible_editor = true;
     if (url.find (editusfm_index_url ()) != string::npos) is_bible_editor = true;
     if (is_bible_editor) {
       bible_editor_count++;
       if (bible_editor_count > 1) {
-        Database_Logs::log ("Setting Bible editor " + url + " read-only");
-        url = filter_url_build_http_query (url, workspace_query_key_readonly (), "");
-        element.second = url;
+        Database_Logs::log ("Setting Bible editor " + url + " as editor number " + convert_to_string (bible_editor_count));
       }
+      editor_numbers [element.first] = bible_editor_count;
     }
   }
+  return editor_numbers;
 }
+
